# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import base64
import json
import re

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type

re_chapter_date = re.compile(r'\d{4}.\d{2}.\d{2}')

# FoOlSlide Open Source online comic management software (NO LONGER MAINTAINED)
# https://github.com/FoolCode/FoOlSlide or https://github.com/chocolatkey/FoOlSlide2 (fork)

# Supported servers:
# Death Toll Reader [EN]
# Jaimini's Box [EN] (disabled)
# Juin Jutsu Team [it]
# Kirei Cake [EN] (disabled)
# Le Cercle du Scan [FR] (disabled)
# NIFTeam [it]


class FoOlSlide(Server):
    base_url: str
    search_url: str
    manga_list_url: str
    manga_url: str
    chapter_url: str
    latest_updates_url = None

    name_selector = '.title'
    cover_selector = '.thumbnail img'
    details_selector = '.info b'
    synopsis_selector = None
    chapters_list_selector = '.list .element'
    chapters_list_link_selector = '.title > a'
    latest_updates_list_selector = '.group > .title > a'
    search_list_selector = '.group'
    search_last_chapter_selector = '.element .title a'
    search_list_cover_selector = 'img.preview'

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers.update({'user-agent': USER_AGENT})

        if self.latest_updates_url is None:
            self.latest_updates_url = self.base_url

    @staticmethod
    def decrypt(value):
        """
        Decrypt a string with a circular shift of 13 of [a-zA-Z] characters

        nopqrstuvwxyzabcdefghijklm => abcdefghijklmnopqrstuvwxyz
        """
        dvalue = ''

        for char in value:
            code = ord(char)
            if 65 <= code <= 77 or 97 <= code <= 109:
                dvalue += chr(code + 13)
            elif 78 <= code <= 90 or 110 <= code <= 122:
                dvalue += chr(code - 13)
            else:
                dvalue += char

        return dvalue

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        adult_alert = False
        if soup.find('div', class_='alert'):
            adult_alert = True

            r = self.session_post(self.manga_url.format(initial_data['slug']), data=dict(adult='true'))
            if r is None:
                return None

            soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[self.name, ],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        data['name'] = soup.select_one(self.name_selector).text.strip()
        if element := soup.select_one(self.cover_selector):
            data['cover'] = element.get('src')

        # Details
        if self.details_selector:
            for element in soup.select(self.details_selector):
                label = element.text
                value = list(element.next_siblings)[0][2:]
                if label in ('Author', 'Artist'):
                    data['authors'].append(value)
                elif label in ('Description', 'Synopsis', 'Trama'):
                    if adult_alert:
                        data['synopsis'] = '{0}\n\n{1}'.format(
                            'ALERT: This series contains mature contents and is meant to be viewed by an adult audience.',
                            value
                        )
                    else:
                        data['synopsis'] = value

        if self.synopsis_selector:
            data['synopsis'] = soup.select_one(self.synopsis_selector).text.strip()

        # Chapters
        for element in reversed(soup.select(self.chapters_list_selector)):
            a_element = element.select_one(self.chapters_list_link_selector)

            title = a_element.text.strip()
            slug = a_element.get('href').split(f'{self.lang}/')[-1][:-1]

            date_match = re.search(re_chapter_date, element.find('div', class_='meta_r').text)
            if date_match:
                date = convert_date_string(date_match.group(), '%Y.%m.%d')
            else:
                date = None

            data['chapters'].append(dict(
                slug=slug,
                date=date,
                title=title,
            ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_post(
            self.chapter_url.format(manga_slug, chapter_slug),
            data=dict(adult='true'),
            headers={
                'Accept-Encoding': 'gzip, deflate',
                'Accept-Language': '{0}-{1},{0};q=0.9,en-US;q=0.8,en;q=0.7'.format(self.lang, self.lang.upper()),
            }
        )
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        # List of pages is available in JavaScript variable '_0x3320' or 'pages'
        # Walk in all scripts to find it
        pages = None
        for script_element in soup.find_all('script'):
            script = script_element.string
            if script is None:
                continue

            for line in script.split('\n'):
                line = line.strip()
                if line.startswith('var _0x3320'):
                    #
                    # Jaimini's Box
                    #
                    pages = line.split(';')[0].split(',')[1][1:-2]
                    # String is encrypted with a circular shift of 13 for [a-zA-Z] characters
                    pages = self.decrypt(pages)
                    # String is BASE64 encoded
                    pages = base64.b64decode(pages)
                    break

                if line.startswith('var pages'):
                    pages = line.replace('var pages = ', '')[:-1]
                    break

            if pages is not None:
                pages = json.loads(pages)
                break

        if pages is None:
            return None

        data = dict(
            pages=[],
        )
        for page in pages:
            data['pages'].append(dict(
                slug=None,
                image=page['url'],
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(page['image'])
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('?')[0].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_manga_list(self, page=1):
        r = self.session_get('{0}/{1}'.format(self.manga_list_url, page))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select(self.search_list_selector):
            a_element = element.find('div', class_='title').a
            if self.search_list_cover_selector:
                cover_element = element.select_one(self.search_list_cover_selector)
            else:
                cover_element = None
            last_chapter_element = element.select_one(self.search_last_chapter_selector)

            results.append(dict(
                slug=a_element.get('href').split('/')[-2],
                name=a_element.get('title'),
                cover=cover_element.get('src') if cover_element else None,
                last_chapter=last_chapter_element.text.strip() if last_chapter_element else None,
            ))

        return results

    def get_latest_updates(self):
        """
        Returns latest updates
        """
        r = self.session_get(self.latest_updates_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        slugs = []
        for a_element in soup.select(self.latest_updates_list_selector):
            slug = a_element.get('href').split('/')[-2]
            if slug in slugs:
                continue
            results.append(dict(
                slug=slug,
                name=a_element.text.strip(),
            ))
            slugs.append(slug)

        return results

    def get_most_populars(self):
        """
        Returns list of all mangas
        """
        r = self.session_get(self.manga_list_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        nav_buttons = soup.find_all('a', class_='gbutton')
        if nav_buttons:
            nb_pages = int(nav_buttons[0].get('href').split('/')[-2])
        else:
            nb_pages = 1

        results = []
        for index in range(nb_pages):
            results += self.get_manga_list(page=index + 1)

        return results

    def search(self, term):
        r = self.session_post(
            self.search_url,
            data=dict(search=term),
            headers={
                'Accept-Encoding': 'gzip, deflate',
                'Accept-Language': '{0}-{1},{0};q=0.9,en-US;q=0.8,en;q=0.7'.format(self.lang, self.lang.upper()),
            }
        )
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')
        results = []
        for element in soup.select(self.search_list_selector):
            a_element = element.find_all('div')[0].a
            if self.search_list_cover_selector:
                cover_element = element.select_one(self.search_list_cover_selector)
            else:
                cover_element = None
            last_chapter_element = element.select_one(self.search_last_chapter_selector)

            results.append(dict(
                slug=a_element.get('href').split('/')[-2],
                name=element.select_one('.title a').text.strip(),
                cover=cover_element.get('src') if cover_element else None,
                last_chapter=last_chapter_element.text.strip() if last_chapter_element else None,
            ))

        return results



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import gc
from gettext import gettext as _
import threading

from gi.repository import GLib

from komikku.explorer.common import ExplorerSearchResultRow
from komikku.explorer.common import ExplorerSearchStackPage
from komikku.models import Settings
from komikku.utils import log_error_traceback


class ExplorerSearchStackPageLatestUpdates(ExplorerSearchStackPage):
    __gtype_name__ = 'ExplorerSearchStackPageLatestUpdates'

    def __init__(self, parent):
        ExplorerSearchStackPage.__init__(self, parent)

        self.stack = self.parent.latest_updates_stack
        self.listbox = self.parent.latest_updates_listbox
        self.no_results_status_page = self.parent.latest_updates_no_results_status_page

        self.listbox.connect('row-activated', self.parent.on_manga_clicked)

        self.no_results_status_page.get_child().connect('clicked', self.populate)

    def populate(self, *args):
        def run(server):
            self.parent.register_request('latest_updates')

            try:
                if results := server.get_latest_updates(**self.parent.search_filters):
                    GLib.idle_add(complete, results, server)
                else:
                    GLib.idle_add(error, results, server)
            except Exception as e:
                GLib.idle_add(error, None, server, *log_error_traceback(e))
            finally:
                gc.collect()

        def complete(results, server):
            if not self.parent.can_page_be_updated_with_results('latest_updates', server.id):
                return

            self.listbox.set_visible(True)

            for item in results:
                row = ExplorerSearchResultRow(item)
                self.listbox.append(row)
                if row.has_cover:
                    self.queue.put((row, server))

            self.stack.set_visible_child_name('results')

            self.render_covers()

        def error(results, server, message=None, stack_trace=None):
            if stack_trace is not None and Settings.get_default().servers_bug_report:
                context = '\n'.join([
                    'Failed to fetch Latest Updates',
                    f'- Server ID: {server.id}',
                    f'- Server URL: {server.base_url}',
                ])
                self.window.open_server_bug_report_dialog(server, context, stack_trace)

            if not self.parent.can_page_be_updated_with_results('latest_updates', server.id):
                return

            if results is None:
                self.no_results_status_page.set_title(_('Oops, failed to retrieve latest updates.'))
                if message:
                    self.no_results_status_page.set_description(message)
            else:
                self.no_results_status_page.set_title(_('No Latest Updates Found'))

            self.stack.set_visible_child_name('no_results')

        self.clear()
        self.stack.set_visible_child_name('loading')

        if self.parent.requests.get('latest_updates') and self.parent.server.id in self.parent.requests['latest_updates']:
            self.window.add_notification(_('A request is already in progress.'), timeout=2)
            return

        thread = threading.Thread(target=run, args=(self.parent.server, ))
        thread.daemon = True
        thread.start()



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.madara import Madara


class Phoenixfansub(Madara):
    id = 'phoenixfansub'
    name = 'Phoenix Fansub'
    lang = 'es'
    status = 'disabled'  # 2023-09

    base_url = 'https://phoenixmangas.com'
    chapters_url = base_url + '/manga/{0}/ajax/chapters/'



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import json

from gi.repository import Gio
from gi.repository import GLib

from komikku.consts import PROGRESSBAR_THEMES


class Settings(Gio.Settings):
    """
    Gio.Settings handler
    Implements the basic dconf-settings as properties
    """

    # Default Settings instance
    instance = None

    def __init__(self):
        super().__init__()

    @staticmethod
    def new():
        """Create a new Settings object"""
        app = Gio.Application.get_default()
        if app is None:
            return None

        g_settings = Gio.Settings.new(app.application_id)
        g_settings.__class__ = Settings

        return g_settings

    @staticmethod
    def get_default():
        """Return the default instance of Settings"""
        if Settings.instance is None:
            Settings.instance = Settings.new()

        return Settings.instance

    @property
    def background_color(self):
        """Return the reader's background color"""
        value = self.background_color_value

        if value == 0:
            return 'white'
        if value == 1:
            return 'black'
        if value == 2:
            return 'gray'
        if value == 3:
            return 'system-style'

    @property
    def background_color_value(self):
        """Return the reader's background color value"""
        return self.get_enum('background-color')

    @background_color.setter
    def background_color(self, color):
        """
        Set the reader's background color

        :param color: reader's background color
        :type color: string
        """
        if color == 'white':
            self.set_enum('background-color', 0)
        elif color == 'black':
            self.set_enum('background-color', 1)
        elif color == 'gray':
            self.set_enum('background-color', 2)
        elif color == 'system-style':
            self.set_enum('background-color', 3)

    @property
    def borders_crop(self):
        return self.get_boolean('borders-crop')

    @borders_crop.setter
    def borders_crop(self, state):
        self.set_boolean('borders-crop', state)

    @property
    def borders_crop_threshold(self):
        """Return broders crop threshold"""
        return self.get_int('borders-crop-threshold')

    @borders_crop_threshold.setter
    def borders_crop_threshold(self, value):
        """
        Set the threshold to classified pixels as `white` during borders detection

        :param value: threshold
        :type value: int
        """
        threshold = GLib.Variant('i', value)
        self.set_value('borders-crop-threshold', threshold)

    @property
    def card_backdrop_method(self):
        """Return the card's backdrop method """
        value = self.card_backdrop_method_value

        if value == 0:
            return None
        if value == 1:
            return 'linear-gradient'
        if value == 2:
            return 'blurred-cover'

    @property
    def card_backdrop_method_value(self):
        """Return the card's backdrop method value"""
        value = self.get_enum('card-backdrop-method')

        if value == -1:
            # Before 1.79.0, value was a boolean and key was `card-backdrop`
            # Convert old value if value is unset (-1)
            old_value = self.get_boolean('card-backdrop')
            value = 1 if old_value else 0

        return value

    @card_backdrop_method.setter
    def card_backdrop_method(self, method):
        """
        Set the card's backdrop method

        :param method: card's backdrop method
        :type method: string
        """
        if method == 'none':
            self.set_enum('card-backdrop-method', 0)
        elif method == 'linear-gradient':
            self.set_enum('card-backdrop-method', 1)
        elif method == 'blurred-cover':
            self.set_enum('card-backdrop-method', 2)

    @property
    def clamp_size(self):
        """Return Webtoon pager clamp size"""
        return self.get_int('clamp-size')

    @clamp_size.setter
    def clamp_size(self, size):
        """
        Set the clamp size of Webtoon pager

        :param size: width in pixels
        :type size: int
        """
        size = GLib.Variant('i', size)
        self.set_value('clamp-size', size)

    @property
    def clear_cached_data_on_app_close(self):
        return self.get_boolean('clear-cached-data-on-app-close')

    @clear_cached_data_on_app_close.setter
    def clear_cached_data_on_app_close(self, state):
        self.set_boolean('clear-cached-data-on-app-close', state)

    @property
    def color_scheme(self):
        """Return the color scheme"""
        value = self.color_scheme_value

        if value == 0:
            return 'light'
        if value == 1:
            return 'dark'
        if value == 2:
            return 'default'

    @property
    def color_scheme_value(self):
        """Return the color scheme value"""
        return self.get_enum('color-scheme')

    @color_scheme.setter
    def color_scheme(self, scheme):
        """
        Set the color scheme

        :param scheme: a color scheme
        :type scheme: string
        """
        if scheme == 'light':
            self.set_enum('color-scheme', 0)
        elif scheme == 'dark':
            self.set_enum('color-scheme', 1)
        elif scheme == 'default':
            self.set_enum('color-scheme', 2)

    @property
    def credentials_storage_plaintext_fallback(self):
        return self.get_boolean('credentials-storage-plaintext-fallback')

    @credentials_storage_plaintext_fallback.setter
    def credentials_storage_plaintext_fallback(self, state):
        self.set_boolean('credentials-storage-plaintext-fallback', state)

    @property
    def disable_animations(self):
        return self.get_boolean('disable-animations')

    @disable_animations.setter
    def disable_animations(self, state):
        self.set_boolean('disable-animations', state)

    @property
    def downloader_state(self):
        return self.get_boolean('downloader-state')

    @downloader_state.setter
    def downloader_state(self, state):
        self.set_boolean('downloader-state', state)

    @property
    def desktop_notifications(self):
        return self.get_boolean('desktop-notifications')

    @desktop_notifications.setter
    def desktop_notifications(self, state):
        self.set_boolean('desktop-notifications', state)

    @property
    def explorer_search_global_selected_filters(self):
        return set(self.get_value('explorer-search-global-selected-filters'))

    @explorer_search_global_selected_filters.setter
    def explorer_search_global_selected_filters(self, filters):
        filters = GLib.Variant('as', filters)
        self.set_value('explorer-search-global-selected-filters', filters)

    @property
    def fullscreen(self):
        return self.get_boolean('fullscreen')

    @fullscreen.setter
    def fullscreen(self, state):
        self.set_boolean('fullscreen', state)

    @property
    def landscape_zoom(self):
        return self.get_boolean('landscape-zoom')

    @landscape_zoom.setter
    def landscape_zoom(self, state):
        self.set_boolean('landscape-zoom', state)

    @property
    def library_badges(self):
        return list(self.get_value('library-badges'))

    @library_badges.setter
    def library_badges(self, badges):
        badges = GLib.Variant('as', badges)
        self.set_value('library-badges', badges)

    @property
    def library_display_mode(self):
        """Return the library's display mode"""
        value = self.library_display_mode_value

        if value == 0:
            return 'grid'
        if value == 1:
            return 'grid-compact'

    @property
    def library_display_mode_value(self):
        """Return the library's display mode value"""
        return self.get_enum('library-display-mode')

    @library_display_mode.setter
    def library_display_mode(self, mode):
        """
        Set the library's display mode

        :param mode: library's display mode
        :type mode: string
        """
        if mode == 'grid':
            self.set_enum('library-display-mode', 0)
        elif mode == 'grid-compact':
            self.set_enum('library-display-mode', 1)

    @property
    def library_selected_filters(self):
        return set(self.get_value('library-selected-filters'))

    @library_selected_filters.setter
    def library_selected_filters(self, filters):
        filters = GLib.Variant('as', filters)
        self.set_value('library-selected-filters', filters)

    @property
    def library_servers_logo(self):
        return self.get_boolean('library-servers-logo')

    @library_servers_logo.setter
    def library_servers_logo(self, state):
        self.set_boolean('library-servers-logo', state)

    @property
    def library_sort_order(self):
        """Return the library's sort order"""
        value = self.library_sort_order_value

        if value == 0:
            return 'alphanum-asc'
        if value == 3:
            return 'latest-read-desc'
        if value == 5:
            return 'latest-updated-desc'

    @property
    def library_sort_order_value(self):
        """Return the library's sort order value"""
        return self.get_enum('library-sort-order')

    @library_sort_order.setter
    def library_sort_order(self, sort):
        """
        Set the library's sort order

        :param sort: library's sort order
        :type sort: string
        """
        if sort == 'alphanum-asc':
            self.set_enum('library-sort-order', 0)
        elif sort == 'latest-read-desc':
            self.set_enum('library-sort-order', 3)
        elif sort == 'latest-updated-desc':
            self.set_enum('library-sort-order', 5)

    @property
    def long_strip_detection(self):
        return self.get_boolean('long-strip-detection')

    @long_strip_detection.setter
    def long_strip_detection(self, state):
        self.set_boolean('long-strip-detection', state)

    @property
    def new_chapters_auto_download(self):
        return self.get_boolean('new-chapters-auto-download')

    @new_chapters_auto_download.setter
    def new_chapters_auto_download(self, state):
        self.set_boolean('new-chapters-auto-download', state)

    @property
    def night_light(self):
        return self.get_boolean('night-light')

    @night_light.setter
    def night_light(self, state):
        self.set_boolean('night-light', state)

    @property
    def nsfw_content(self):
        return self.get_boolean('nsfw-content')

    @nsfw_content.setter
    def nsfw_content(self, state):
        self.set_boolean('nsfw-content', state)

    @property
    def nsfw_only_content(self):
        return self.get_boolean('nsfw-only-content')

    @nsfw_only_content.setter
    def nsfw_only_content(self, state):
        self.set_boolean('nsfw-only-content', state)

    @property
    def ocr_lang(self):
        return self.get_string('ocr-lang')

    @ocr_lang.setter
    def ocr_lang(self, lang):
        self.set_string('ocr-lang', lang)

    @property
    def page_filters(self):
        return json.loads(self.get_string('page-filters'))

    @page_filters.setter
    def page_filters(self, state):
        self.set_string('page-filters', json.dumps(state))

    @property
    def page_numbering(self):
        return self.get_boolean('page-numbering')

    @page_numbering.setter
    def page_numbering(self, state):
        self.set_boolean('page-numbering', state)

    def add_pinned_server(self, id):
        ids = self.pinned_servers
        if id not in ids:
            ids.append(id)

        self.pinned_servers = ids

    def remove_pinned_server(self, id):
        ids = self.pinned_servers
        if id in ids:
            ids.remove(id)

        self.pinned_servers = ids

    @property
    def pinned_servers(self):
        return list(self.get_value('pinned-servers'))

    @pinned_servers.setter
    def pinned_servers(self, ids):
        ids = GLib.Variant('as', ids)
        self.set_value('pinned-servers', ids)

    @property
    def progressbar_theme(self):
        """Return progressbar theme"""
        return list(PROGRESSBAR_THEMES.keys())[self.progressbar_theme_value]

    @property
    def progressbar_theme_value(self):
        """Return progressbar theme value"""
        return self.get_enum('progressbar-theme')

    @progressbar_theme.setter
    def progressbar_theme(self, value):
        """
        Set progressbar theme

        :param value: progressbar theme
        :type value: string
        """
        self.set_enum('progressbar-theme', list(PROGRESSBAR_THEMES.keys()).index(value))

    @property
    def reading_mode(self):
        """Return the reader's reading mode"""
        value = self.reading_mode_value

        if value == 0:
            return 'right-to-left'
        if value == 1:
            return 'left-to-right'
        if value == 2:
            return 'vertical'
        if value == 3:
            return 'webtoon'

    @property
    def reading_mode_value(self):
        """Return the reader's reading mode value"""
        mode = -1
        try:
            # Before 0.22.0 reading mode was called reading direction
            mode = self.get_enum('reading-direction')
        except Exception:
            pass  # noqa: TC202

        if mode < 0:
            mode = self.get_enum('reading-mode')

        return mode

    @reading_mode.setter
    def reading_mode(self, mode):
        """
        Set the reader's reading mode

        :param mode: reader's reading mode
        :type mode: string
        """
        try:
            # Before 0.22.0 reading mode was called reading direction
            # Clear previous key with -1 value (unused)
            self.get_enum('reading-direction')
            self.set_enum('reading-direction', -1)
        except Exception:
            pass  # noqa: TC202

        if mode == 'right-to-left':
            self.set_enum('reading-mode', 0)
        elif mode == 'left-to-right':
            self.set_enum('reading-mode', 1)
        elif mode == 'vertical':
            self.set_enum('reading-mode', 2)
        elif mode == 'webtoon':
            self.set_enum('reading-mode', 3)

    @property
    def scaling(self):
        """Return the pages' scaling in reader"""
        value = self.scaling_value

        if value == 0:
            return 'screen'
        if value == 1:
            return 'width'
        if value == 2:
            return 'height'
        if value == 3:
            return 'original'

    @property
    def scaling_value(self):
        """Return the pages' scaling value in reader"""
        return self.get_enum('scaling')

    @scaling.setter
    def scaling(self, value):
        """
        Set the pages' scaling in reader

        :param value: pages' scaling
        :type value: string
        """
        if value == 'screen':
            self.set_enum('scaling', 0)
        elif value == 'width':
            self.set_enum('scaling', 1)
        elif value == 'height':
            self.set_enum('scaling', 2)
        elif value == 'original':
            self.set_enum('scaling', 3)

    @property
    def scaling_filter(self):
        """Return the pages' scaling filter in reader"""
        value = self.scaling_filter_value

        if value == 0:
            return 'linear'
        if value == 1:
            return 'trilinear'

    @property
    def scaling_filter_value(self):
        """Return the pages' scaling filter value in reader"""
        return self.get_enum('scaling-filter')

    @scaling_filter.setter
    def scaling_filter(self, value):
        """
        Set the pages' scaling filter in reader

        :param value: pages' scaling_filter
        :type value: string
        """
        if value == 'linear':
            self.set_enum('scaling-filter', 0)
        elif value == 'trilinear':
            self.set_enum('scaling-filter', 1)

    @property
    def scroll_click_percentage(self):
        """Return scroll click distance percentage for Webtoon pager"""
        return self.get_double('scroll-click-percentage')

    @scroll_click_percentage.setter
    def scroll_click_percentage(self, percentage):
        """
        Set the scroll click distance percentage for Webtoon pager

        :param percentage: percentage of page height to scroll (0.0 to 1.0)
        :type percentage: float
        """
        percentage = GLib.Variant('d', percentage)
        self.set_value('scroll-click-percentage', percentage)

    @property
    def scroll_drag_factor(self):
        """Return scroll drag sensitivity factor for Webtoon pager"""
        return self.get_double('scroll-drag-factor')

    @scroll_drag_factor.setter
    def scroll_drag_factor(self, factor):
        """
        Set the scroll drag sensitivity factor for Webtoon pager

        :param factor: sensitivity multiplier
        :type factor: float
        """
        factor = GLib.Variant('d', factor)
        self.set_value('scroll-drag-factor', factor)

    @property
    def selected_category(self):
        return self.get_int('selected-category')

    @selected_category.setter
    def selected_category(self, state):
        self.set_int('selected-category', state)

    @property
    def servers_bug_report(self):
        return self.get_boolean('servers-bug-report')

    @servers_bug_report.setter
    def servers_bug_report(self, state):
        self.set_boolean('servers-bug-report', state)

    @property
    def servers_external_modules(self):
        return self.get_boolean('external-servers-modules')

    @servers_external_modules.setter
    def servers_external_modules(self, state):
        self.set_boolean('external-servers-modules', state)

    def toggle_server(self, uid, state):
        settings = self.servers_settings

        if uid not in settings:
            settings[uid] = {
                'langs': {},
            }

        settings[uid]['enabled'] = state

        self.servers_settings = settings

    def toggle_server_lang(self, uid, lang, state):
        settings = self.servers_settings

        if uid not in settings:
            settings[uid] = {
                'langs': {},
                'enabled': True,
            }

        settings[uid]['langs'][lang] = state

        self.servers_settings = settings

    @property
    def servers_settings(self):
        return json.loads(self.get_string('servers-settings'))

    @servers_settings.setter
    def servers_settings(self, state):
        self.set_string('servers-settings', json.dumps(state))

    def add_servers_language(self, code):
        codes = self.servers_languages
        if code not in codes:
            codes.append(code)

        self.servers_languages = codes

    def remove_servers_language(self, code):
        codes = self.servers_languages
        if code in codes:
            codes.remove(code)

        self.servers_languages = codes

    @property
    def servers_languages(self):
        return list(self.get_value('servers-languages'))

    @servers_languages.setter
    def servers_languages(self, codes):
        codes = GLib.Variant('as', codes)
        self.set_value('servers-languages', codes)

    @property
    def system_accent_colors(self):
        return self.get_boolean('system-accent-colors')

    @system_accent_colors.setter
    def system_accent_colors(self, state):
        self.set_boolean('system-accent-colors', state)

    @property
    def trackers(self):
        return json.loads(self.get_string('trackers'))

    @trackers.setter
    def trackers(self, state):
        self.set_string('trackers', json.dumps(state))

    @property
    def tracking(self):
        return self.get_boolean('tracking')

    @tracking.setter
    def tracking(self, state):
        self.set_boolean('tracking', state)

    @property
    def update_at_startup(self):
        return self.get_boolean('update-at-startup')

    @update_at_startup.setter
    def update_at_startup(self, state):
        self.set_boolean('update-at-startup', state)

    @property
    def window_maximized_state(self):
        return self.get_boolean('window-maximized-state')

    @window_maximized_state.setter
    def window_maximized_state(self, state):
        self.set_boolean('window-maximized-state', state)

    @property
    def window_size(self):
        """Return the window's size"""
        return tuple(self.get_value('window-size'))

    @window_size.setter
    def window_size(self, size):
        """
        Set the window's size

        :param size: [w, h] window's size
        :type size: list
        """
        size = GLib.Variant('ai', list(size))
        self.set_value('window-size', size)



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import base64
import logging
import time
from urllib.parse import urlparse

from bs4 import BeautifulSoup
from cryptography.hazmat.primitives.ciphers import algorithms
from cryptography.hazmat.primitives.ciphers import Cipher
from cryptography.hazmat.primitives.ciphers import modes
try:
    # This server requires JA3/TLS and HTTP2 fingerprints impersonation
    from curl_cffi import requests
except Exception:
    # Server will be disabled
    requests = None
import unidecode

from komikku.consts import REQUESTS_TIMEOUT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.servers.utils import sojson4_decode
from komikku.utils import get_buffer_mime_type

SEARCH_RESULTS_PAGES = 5
MOST_POPULAR_RESULTS_PAGES = 2
LATEST_UPDATES_RESULTS_PAGES = 5

logger = logging.getLogger('komikku.servers.mangago')


class Mangago(Server):
    id = 'mangago'
    name = 'Mangago'
    lang = 'en'
    status = 'enabled' if requests is not None else 'disabled'

    http_client = 'curl_cffi'

    base_url = 'https://www.mangago.me'
    logo_url = base_url + '/ywz.ico'
    search_url = base_url + '/r/l_search/'
    latest_updates_url = base_url + '/list/latest/all/{0}/'
    most_populars_url = base_url + '/topmanga/'
    manga_url = base_url + '/read-manga/{0}/'

    def __init__(self):
        if self.session is None and requests is not None:
            self.session = requests.Session(allow_redirects=True, impersonate='chrome', timeout=(REQUESTS_TIMEOUT, REQUESTS_TIMEOUT * 2))

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        data['name'] = soup.select_one('h1').text.strip()
        data['cover'] = soup.select_one('.cover img').get('src')

        # Details
        for element in soup.select('.manga_right td'):
            label = element.label.text.strip()

            if label.startswith('Author'):
                for a_element in element.select('a'):
                    author = a_element.text.strip()
                    if not author:
                        continue
                    data['authors'].append(author)

            elif label.startswith('Genre'):
                for a_element in element.select('a'):
                    genre = a_element.text.strip()

                    if f'({genre})' in data['name']:
                        data['name'] = data['name'].replace(f'({genre})', '').strip()

                    data['genres'].append(genre)

            elif label.startswith('Status'):
                value = element.span.text.strip().lower()

                if value == 'ongoing':
                    data['status'] = 'ongoing'
                elif value == 'completed':
                    data['status'] = 'complete'

        # Synopsis
        if synopsis_element := soup.select_one('.manga_summary'):
            data['synopsis'] = synopsis_element.text.strip()

        # Chapters
        manga_url = self.manga_url.format(data['slug'])
        for tr_element in reversed(soup.select('#chapter_table tr')):
            td_elements = tr_element.select('td')
            slug_element = td_elements[0].select_one('a.chico')

            url = slug_element.get('href')
            if url.startswith(manga_url):
                slug = url.replace(manga_url, '')
            else:
                logger.warning('Chapter ignored: url unknown %s', url)
                continue

            data['chapters'].append(dict(
                slug=slug,
                url=url,
                title=unidecode.unidecode(slug_element.text.strip()),
                date=convert_date_string(td_elements[2].text.strip(), format='%b %d, %Y'),
            ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url, data=None):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(chapter_url)
        if r.status_code != 200:
            return None

        if r.redirect_count:
            return data

        soup = BeautifulSoup(r.text, 'lxml')

        # Get page URLs list: AES encryption + base64 encoding
        imgsrcs = None
        for script_element in soup.select('script'):
            script = script_element.string
            if not script or 'var imgsrcs' not in script:
                continue

            for line in script.split('\n'):
                line = line.strip()
                if line.startswith('var imgsrcs'):
                    imgsrcs = line.split(' = ')[1][1:-2]
                    break

            break

        if imgsrcs is None:
            logger.warning('Failed to get encrypted pages URLs list')
            return None

        # Get URL of JS file in charge of decryption (sojson.v4 obfuscated)
        chapterjs_url = None
        for script_element in soup.select('script'):
            src = script_element.get('src')
            if src and 'chapter.js' in src:
                chapterjs_url = src
                break

        if chapterjs_url is None:
            logger.warning('Failed to get URL of JS file in charge of decryption')
            return None

        # Get file and deobfuscate it
        r = self.session_get(chapterjs_url)
        if 'sojson.v4' not in r.text:
            logger.warning('JS file in charge of decryption is not obfuscated with sojson.v4!')
            return None

        js_code = sojson4_decode(r.text)

        # Parse JS code to get AES key and iv (both in hexadecimal)
        key = None
        iv = None
        for line in js_code.split('\n'):
            if 'var key = CryptoJS' in line:
                key = line.split('"')[-2]
            if 'var iv  = CryptoJS' in line:
                iv = line.split('"')[-2]
            if iv and key:
                break

        if key is None or iv is None:
            logger.warning('Failed to get AES key and/or iv')
            return None

        key = bytes.fromhex(key)
        iv = bytes.fromhex(iv)
        imgsrcs = base64.b64decode(imgsrcs)

        # Decrypt
        cipher = Cipher(algorithms.AES(key), modes.CBC(iv))
        decryptor = cipher.decryptor()
        imgsrcs = decryptor.update(imgsrcs) + decryptor.finalize()
        imgsrcs = imgsrcs.decode('utf-8').rstrip('\x00').split(',')

        if data is None:
            data = dict(
                pages=[],
            )

        partial = False
        for url in imgsrcs:
            if not url:
                partial = True
                break

            data['pages'].append(dict(
                slug=None,
                image=url,
            ))

        if partial:
            next_url = soup.select_one('.left a.next_page').get('href')
            if not next_url.startswith('http'):
                parsed = urlparse(chapter_url)
                next_url = f'{parsed.scheme}{parsed.netloc}{next_url}'

            return self.get_manga_chapter_data(manga_slug, manga_name, chapter_slug, next_url, data=data)

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        # Scrap HTML page to get image url
        r = self.session_get(page['image'])
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_latest_updates(self):
        """
        Returns Latest Updates list
        """
        def get_page(page):
            r = self.session_get(self.latest_updates_url.format(page))
            if r.status_code != 200:
                return None, False

            soup = BeautifulSoup(r.text, 'lxml')

            import unidecode

            items = []
            for element in soup.select('#search_list li .box'):
                a_element = element.select_one('a.thm-effect')
                if last_chapter_element := element.select_one('a.chico'):
                    last_chapter = last_chapter_element.text.strip()
                else:
                    last_chapter = None

                items.append(dict(
                    slug=a_element.get('href').split('/')[-2],
                    name=unidecode.unidecode(a_element.get('title')).strip(),
                    cover=a_element.img.get('src'),
                    last_chapter=last_chapter,
                ))

            if pagination := soup.select_one('.pagination'):
                nb_pages = int(pagination.get('total'))
            else:
                nb_pages = 1

            return items, page < min(LATEST_UPDATES_RESULTS_PAGES, nb_pages)

        results = []
        delay = None
        for page in range(1, LATEST_UPDATES_RESULTS_PAGES + 1):
            if delay:
                time.sleep(delay)

            items, more = get_page(page)
            if items is not None:
                results += items
            else:
                return None

            if not more:
                break

            delay = 1

        return results

    def get_most_populars(self):
        """
        Returns most view manga list
        """
        def get_page(page):
            r = self.session_get(
                self.most_populars_url,
                params=dict(
                    f=1,
                    o=1,
                    sortby='view',
                    e='',
                )
            )
            if r.status_code != 200:
                return None, False

            soup = BeautifulSoup(r.text, 'lxml')

            items = []
            for a_element in soup.select('.pic_list .listitem a.thm-effect'):
                items.append(dict(
                    slug=a_element.get('href').split('/')[-2],
                    name=unidecode.unidecode(a_element.get('title')).strip(),
                    cover=a_element.img.get('data-src'),
                ))

            if pagination := soup.select_one('.pagination'):
                nb_pages = int(pagination.get('total'))
            else:
                nb_pages = 1

            return items, page < min(MOST_POPULAR_RESULTS_PAGES, nb_pages)

        results = []
        delay = None
        for page in range(1, MOST_POPULAR_RESULTS_PAGES + 1):
            if delay:
                time.sleep(delay)

            items, more = get_page(page)
            if items is not None:
                results += items
            else:
                return None

            if not more:
                break

            delay = 1

        return results

    def search(self, term):
        def get_page(name, page):
            r = self.session_get(self.search_url, params=dict(name=term, page=page))
            if r.status_code != 200:
                return None, False

            soup = BeautifulSoup(r.text, 'lxml')

            items = []
            for element in soup.select('#search_list li .box'):
                a_element = element.select_one('a.thm-effect')
                if last_chapter_element := element.select_one('a.chico'):
                    last_chapter = last_chapter_element.text.strip()
                else:
                    last_chapter = None

                items.append(dict(
                    slug=a_element.get('href').split('/')[-2],
                    name=unidecode.unidecode(a_element.get('title')).strip(),
                    cover=a_element.img.get('src'),
                    last_chapter=last_chapter,
                ))

            if pagination := soup.select_one('.pagination'):
                nb_pages = int(pagination.get('total'))
            else:
                nb_pages = 1

            return items, page < min(SEARCH_RESULTS_PAGES, nb_pages)

        results = []
        delay = None
        for page in range(1, SEARCH_RESULTS_PAGES + 1):
            if delay:
                time.sleep(delay)

            items, more = get_page(term, page)
            if items is not None:
                results += items
            else:
                return None

            if not more:
                break

            delay = 1

        return results



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gettext import gettext as _

from gi.repository import Adw
from gi.repository import GLib
from gi.repository import Gtk

from komikku.consts import BORDERS_CROP_THRESHOLDS
from komikku.consts import PROGRESSBAR_THEMES
from komikku.models import Settings
from komikku.models.database import clear_cached_data
from komikku.preferences.servers import PreferencesServersLanguagesSubPage
from komikku.preferences.servers import PreferencesServersSettingsSubPage
from komikku.preferences.trackers import TrackerRow
from komikku.utils import folder_size
from komikku.utils import get_cached_data_dir
from komikku.utils import get_webview_data_dir


@Gtk.Template.from_resource('/info/febvre/Komikku/ui/preferences.ui')
class PreferencesDialog(Adw.PreferencesDialog):
    __gtype_name__ = 'PreferencesDialog'

    support_group = Gtk.Template.Child('support_group')
    support_button = Gtk.Template.Child('support_button')
    support_close_button = Gtk.Template.Child('support_close_button')

    color_scheme_row = Gtk.Template.Child('color_scheme_row')
    night_light_switch = Gtk.Template.Child('night_light_switch')
    system_accent_colors_switch = Gtk.Template.Child('system_accent_colors_switch')
    progressbar_theme_row = Gtk.Template.Child('progressbar_theme_row')
    card_backdrop_method_row = Gtk.Template.Child('card_backdrop_method_row')
    desktop_notifications_switch = Gtk.Template.Child('desktop_notifications_switch')
    tracking_group = Gtk.Template.Child('tracking_group')
    tracking_switch = Gtk.Template.Child('tracking_switch')

    library_display_mode_row = Gtk.Template.Child('library_display_mode_row')
    library_servers_logo_switch = Gtk.Template.Child('library_servers_logo_switch')
    library_badge_unread_chapters_switch = Gtk.Template.Child('library_badge_unread_chapters_switch')
    library_badge_downloaded_chapters_switch = Gtk.Template.Child('library_badge_downloaded_chapters_switch')
    library_badge_recent_chapters_switch = Gtk.Template.Child('library_badge_recent_chapters_switch')
    update_at_startup_switch = Gtk.Template.Child('update_at_startup_switch')
    new_chapters_auto_download_switch = Gtk.Template.Child('new_chapters_auto_download_switch')
    nsfw_content_switch = Gtk.Template.Child('nsfw_content_switch')
    nsfw_only_content_switch = Gtk.Template.Child('nsfw_only_content_switch')
    servers_languages_actionrow = Gtk.Template.Child('servers_languages_actionrow')
    servers_settings_actionrow = Gtk.Template.Child('servers_settings_actionrow')
    long_strip_detection_switch = Gtk.Template.Child('long_strip_detection_switch')

    reading_mode_row = Gtk.Template.Child('reading_mode_row')
    scaling_filter_row = Gtk.Template.Child('scaling_filter_row')
    background_color_row = Gtk.Template.Child('background_color_row')
    page_numbering_switch = Gtk.Template.Child('page_numbering_switch')
    fullscreen_switch = Gtk.Template.Child('fullscreen_switch')
    scaling_row = Gtk.Template.Child('scaling_row')
    landscape_zoom_switch = Gtk.Template.Child('landscape_zoom_switch')
    borders_crop_switch = Gtk.Template.Child('borders_crop_switch')
    borders_crop_threshold_row = Gtk.Template.Child('borders_crop_threshold_row')
    webtoon_reading_mode_group_reset_button = Gtk.Template.Child('webtoon_reading_mode_group_reset_button')
    clamp_size_adjustment = Gtk.Template.Child('clamp_size_adjustment')
    scroll_click_percentage_adjustment = Gtk.Template.Child('scroll_click_percentage_adjustment')
    scroll_drag_factor_adjustment = Gtk.Template.Child('scroll_drag_factor_adjustment')
    ocr_lang_row = Gtk.Template.Child('ocr_lang_row')

    advanced_banner = Gtk.Template.Child('advanced_banner')
    clear_cached_data_actionrow = Gtk.Template.Child('clear_cached_data_actionrow')
    clear_cached_data_button = Gtk.Template.Child('clear_cached_data_button')
    clear_cached_data_on_app_close_switch = Gtk.Template.Child('clear_cached_data_on_app_close_switch')
    clear_webview_data_actionrow = Gtk.Template.Child('clear_webview_data_actionrow')
    clear_webview_data_button = Gtk.Template.Child('clear_webview_data_button')
    servers_external_modules_switch = Gtk.Template.Child('servers_external_modules_switch')
    servers_bug_report_switch = Gtk.Template.Child('servers_bug_report_switch')
    credentials_storage_plaintext_fallback_switch = Gtk.Template.Child('credentials_storage_plaintext_fallback_switch')
    disable_animations_switch = Gtk.Template.Child('disable_animations_switch')

    def __init__(self, window):
        Adw.PreferencesDialog.__init__(self)

        self.window = window
        self.settings = Settings.get_default()
        self.servers_external_modules_in_use = self.settings.servers_external_modules

        self.support_button.connect('clicked', lambda _btn: self.push_subpage(self.window.support))
        self.support_close_button.connect('clicked', lambda _btn: self.support_group.set_visible(False))

        self.connect('closed', self.on_closed)
        self.advanced_banner.connect('button-clicked', lambda banner: banner.set_revealed(False))

        self.set_config_values()

    def on_background_color_changed(self, row, _gparam):
        index = row.get_selected()

        if index == 0:
            self.settings.background_color = 'white'
        elif index == 1:
            self.settings.background_color = 'black'
        elif index == 2:
            self.settings.background_color = 'gray'
        elif index == 3:
            self.settings.background_color = 'system-style'

    def on_borders_crop_changed(self, switch_button, _gparam):
        self.settings.borders_crop = switch_button.get_active()

    def on_borders_crop_threshold_changed(self, row, _gparam):
        self.settings.borders_crop_threshold = BORDERS_CROP_THRESHOLDS[row.get_selected()]

    def on_card_backdrop_method_changed(self, row, _gparam):
        index = row.get_selected()

        if index == 0:
            self.settings.card_backdrop_method = 'none'
            self.window.card.remove_backdrop()
        elif index == 1:
            self.settings.card_backdrop_method = 'linear-gradient'
            self.window.card.set_backdrop()
        elif index == 2:
            self.settings.card_backdrop_method = 'blurred-cover'
            self.window.card.set_backdrop()

    def on_closed(self, _dialog):
        # Pop subpage if one is opened when dialog is closed
        self.pop_subpage()
        # Close search if opened
        self.set_search_enabled(False)

    def on_color_scheme_changed(self, row, _gparam):
        index = row.get_selected()

        if index == 0:
            self.settings.color_scheme = 'light'
        elif index == 1:
            self.settings.color_scheme = 'dark'
        elif index == 2:
            self.settings.color_scheme = 'default'

        self.window.init_theme()
        self.window.card.set_backdrop()

    def on_clamp_size_changed(self, adjustment):
        self.settings.clamp_size = int(adjustment.get_value())

    def on_clear_cached_data_clicked(self, _button):
        # Clear cached data of manga not in library
        # If a manga is being read, it must be excluded

        def confirm_callback():
            manga_in_use = None
            if self.window.previous_page in ('card', 'reader') and not self.window.card.manga.in_library:
                manga_in_use = self.window.card.manga

            clear_cached_data(manga_in_use)
            self.update_cached_data_size()

            if self.window.previous_page == 'history':
                self.window.history.populate()

        self.window.open_dialog(
            _('Clear?'),
            body=_('Are you sure you want to clear chapters cache and database?'),
            confirm_label=_('Clear'),
            confirm_callback=confirm_callback,
            confirm_appearance=Adw.ResponseAppearance.DESTRUCTIVE
        )

    def on_clear_cached_data_on_app_close_changed(self, switch_button, _gparam):
        self.settings.clear_cached_data_on_app_close = switch_button.get_active()

    def on_clear_webview_data_clicked(self, _button):
        # Clear WebView data

        def confirm_callback():
            self.window.webview.clear_data(on_clear_data_finished)

        def on_clear_data_finished(success):
            if success:
                self.update_webview_data_size()
            else:
                self.add_toast(Adw.Toast.new(_('Failed to clear WebView data')))

        self.window.open_dialog(
            _('Clear?'),
            body=_('Are you sure you want to clear WebView data (cache, storage, cookies)?'),
            confirm_label=_('Clear'),
            confirm_callback=confirm_callback,
            confirm_appearance=Adw.ResponseAppearance.DESTRUCTIVE
        )

    def on_credentials_storage_plaintext_fallback_changed(self, switch_button, _gparam):
        self.settings.credentials_storage_plaintext_fallback = switch_button.get_active()

    def on_desktop_notifications_changed(self, switch_button, _gparam):
        if switch_button.get_active():
            self.settings.desktop_notifications = True
        else:
            self.settings.desktop_notifications = False

    def on_disable_animations_changed(self, switch_button, _gparam):
        def on_cancel():
            switch_button.set_active(False)

        def on_confirm():
            self.settings.disable_animations = True
            Gtk.Settings.get_default().set_property('gtk-enable-animations', False)

        if switch_button.get_active():
            self.window.open_dialog(
                _('Disable animations?'),
                body=_('Are you sure you want to disable animations?\n\nThe gesture navigation in the reader will not work properly anymore.'),
                confirm_label=_('Disable'),
                confirm_callback=on_confirm,
                confirm_appearance=Adw.ResponseAppearance.DESTRUCTIVE,
                cancel_callback=on_cancel
            )
        elif self.settings.disable_animations:
            self.settings.disable_animations = False
            Gtk.Settings.get_default().set_property('gtk-enable-animations', True)

    def on_fullscreen_changed(self, switch_button, _gparam):
        self.settings.fullscreen = switch_button.get_active()

    def on_landscape_zoom_changed(self, switch_button, _gparam):
        self.settings.landscape_zoom = switch_button.get_active()

    def on_library_badge_changed(self, switch_button, _gparam):
        badges = self.settings.library_badges
        if switch_button.get_active():
            if switch_button._value not in badges:
                badges.append(switch_button._value)
        else:
            if switch_button._value in badges:
                badges.remove(switch_button._value)
        self.settings.library_badges = badges

        GLib.idle_add(self.window.library.populate)

    def on_library_display_mode_changed(self, row, _gparam):
        index = row.get_selected()

        if index == 0:
            self.settings.library_display_mode = 'grid'
        elif index == 1:
            self.settings.library_display_mode = 'grid-compact'

        GLib.idle_add(self.window.library.populate)

    def on_library_servers_logo_changed(self, switch_button, _gparam):
        if switch_button.get_active():
            self.settings.library_servers_logo = True
        else:
            self.settings.library_servers_logo = False

        GLib.idle_add(self.window.library.populate)

    def on_long_strip_detection_changed(self, switch_button, _gparam):
        self.settings.long_strip_detection = switch_button.get_active()

    def on_progressbar_theme_changed(self, row, _gparam):
        self.settings.progressbar_theme = list(PROGRESSBAR_THEMES.keys())[row.get_selected()]

        self.window.init_progressbar_theme()

    def on_new_chapters_auto_download_changed(self, switch_button, _gparam):
        if switch_button.get_active():
            self.settings.new_chapters_auto_download = True
        else:
            self.settings.new_chapters_auto_download = False

    def on_night_light_changed(self, switch_button, _gparam):
        self.settings.night_light = switch_button.get_active()

        self.window.init_theme()

    def on_nsfw_content_changed(self, switch_button, _gparam):
        if switch_button.get_active():
            self.settings.nsfw_content = True
        else:
            self.settings.nsfw_content = False

    def on_nsfw_only_content_changed(self, switch_button, _gparam):
        if switch_button.get_active():
            self.settings.nsfw_only_content = True
        else:
            self.settings.nsfw_only_content = False

    def on_ocr_lang_changed(self, row, _gparam):
        self.settings.ocr_lang = self.window.reader.ocr_translator.ocr_languages[row.get_selected()]

    def on_page_numbering_changed(self, switch_button, _gparam):
        self.settings.page_numbering = not switch_button.get_active()

    def on_reading_mode_changed(self, row, _gparam):
        index = row.get_selected()

        if index == 0:
            self.settings.reading_mode = 'right-to-left'
        elif index == 1:
            self.settings.reading_mode = 'left-to-right'
        elif index == 2:
            self.settings.reading_mode = 'vertical'
        elif index == 3:
            self.settings.reading_mode = 'webtoon'

    def on_scaling_changed(self, row, _gparam):
        index = row.get_selected()

        if index == 0:
            self.settings.scaling = 'screen'
        elif index == 1:
            self.settings.scaling = 'width'
        elif index == 2:
            self.settings.scaling = 'height'
        elif index == 3:
            self.settings.scaling = 'original'

    def on_scaling_filter_changed(self, row, _gparam):
        index = row.get_selected()

        if index == 0:
            self.settings.scaling_filter = 'linear'
        elif index == 1:
            self.settings.scaling_filter = 'trilinear'

    def on_scroll_click_percentage_changed(self, adjustment):
        self.settings.scroll_click_percentage = adjustment.get_value()

    def on_scroll_drag_factor_changed(self, adjustment):
        self.settings.scroll_drag_factor = adjustment.get_value()

    def on_servers_bug_report_changed(self, switch_button, _gparam):
        self.settings.servers_bug_report = switch_button.get_active()

    def on_servers_external_modules_changed(self, switch_button, _gparam):
        active = switch_button.get_active()
        self.advanced_banner.set_revealed(active != self.servers_external_modules_in_use)
        self.settings.servers_external_modules = active

    def on_system_accent_colors_changed(self, switch_button, _gparam):
        self.settings.system_accent_colors = switch_button.get_active()

        self.window.init_accent_colors()

    def on_tracking_changed(self, switch_button, _gparam):
        self.settings.tracking = switch_button.get_active()

    def on_update_at_startup_changed(self, switch_button, _gparam):
        if switch_button.get_active():
            self.settings.update_at_startup = True
        else:
            self.settings.update_at_startup = False

    def reset_webtoon_reading_mode_settings(self, _btn):
        self.clamp_size_adjustment.set_value(
            self.settings.get_default_value('clamp-size').get_int32()
        )
        self.scroll_click_percentage_adjustment.set_value(
            self.settings.get_default_value('scroll-click-percentage').get_double()
        )
        self.scroll_drag_factor_adjustment.set_value(
            self.settings.get_default_value('scroll-drag-factor').get_double()
        )

    def set_config_values(self):
        #
        # General
        #

        # Theme
        if not Adw.StyleManager.get_default().get_system_supports_color_schemes():
            # System doesn't support color schemes
            self.color_scheme_row.get_model().remove(2)
            if self.settings.color_scheme == 'default':
                self.settings.color_scheme = 'light'
        self.color_scheme_row.set_selected(self.settings.color_scheme_value)
        self.color_scheme_row.connect('notify::selected', self.on_color_scheme_changed)

        # Night light
        self.night_light_switch.set_active(self.settings.night_light)
        self.night_light_switch.connect('notify::active', self.on_night_light_changed)

        # Use system accent colors
        self.system_accent_colors_switch.set_active(self.settings.system_accent_colors)
        self.system_accent_colors_switch.connect('notify::active', self.on_system_accent_colors_changed)

        # Progress bars theme
        model = Gtk.StringList()
        for _key, theme in PROGRESSBAR_THEMES.items():
            model.append(theme['name'])
        self.progressbar_theme_row.set_model(model)
        self.progressbar_theme_row.set_selected(self.settings.progressbar_theme_value)
        self.progressbar_theme_row.connect('notify::selected', self.on_progressbar_theme_changed)

        progressbar = Gtk.ProgressBar(fraction=1, halign=Gtk.Align.START, margin_top=4)
        self.progressbar_theme_row.get_first_child().get_first_child().get_next_sibling().get_next_sibling().append(progressbar)

        # Card backdrop method
        self.card_backdrop_method_row.set_selected(self.settings.card_backdrop_method_value)
        self.card_backdrop_method_row.connect('notify::selected', self.on_card_backdrop_method_changed)

        # Desktop notifications
        self.desktop_notifications_switch.set_active(self.settings.desktop_notifications)
        self.desktop_notifications_switch.connect('notify::active', self.on_desktop_notifications_changed)

        # Tracking
        self.tracking_switch.set_active(self.settings.tracking)
        self.tracking_switch.connect('notify::active', self.on_tracking_changed)

        for _id, tracker in self.window.trackers.trackers.items():
            row = TrackerRow(self.window, tracker)
            self.tracking_group.add(row)

        #
        # Library
        #

        # Display mode
        self.library_display_mode_row.set_selected(self.settings.library_display_mode_value)
        self.library_display_mode_row.connect('notify::selected', self.on_library_display_mode_changed)

        # Servers logo
        self.library_servers_logo_switch.set_active(self.settings.library_servers_logo)
        self.library_servers_logo_switch.connect('notify::active', self.on_library_servers_logo_changed)

        # Badges
        self.library_badge_unread_chapters_switch.set_active('unread-chapters' in self.settings.library_badges)
        self.library_badge_unread_chapters_switch._value = 'unread-chapters'
        self.library_badge_unread_chapters_switch.connect('notify::active', self.on_library_badge_changed)
        self.library_badge_downloaded_chapters_switch.set_active('downloaded-chapters' in self.settings.library_badges)
        self.library_badge_downloaded_chapters_switch._value = 'downloaded-chapters'
        self.library_badge_downloaded_chapters_switch.connect('notify::active', self.on_library_badge_changed)
        self.library_badge_recent_chapters_switch.set_active('recent-chapters' in self.settings.library_badges)
        self.library_badge_recent_chapters_switch._value = 'recent-chapters'
        self.library_badge_recent_chapters_switch.connect('notify::active', self.on_library_badge_changed)

        # Update manga at startup
        self.update_at_startup_switch.set_active(self.settings.update_at_startup)
        self.update_at_startup_switch.connect('notify::active', self.on_update_at_startup_changed)

        # Auto download new chapters
        self.new_chapters_auto_download_switch.set_active(self.settings.new_chapters_auto_download)
        self.new_chapters_auto_download_switch.connect('notify::active', self.on_new_chapters_auto_download_changed)

        # Servers languages
        self.servers_languages_actionrow.props.activatable = True
        self.servers_languages_subpage = PreferencesServersLanguagesSubPage(self)
        self.servers_languages_actionrow.connect('activated', self.servers_languages_subpage.populate)

        # Servers settings
        self.servers_settings_actionrow.props.activatable = True
        self.servers_settings_subpage = PreferencesServersSettingsSubPage(self)
        self.servers_settings_actionrow.connect('activated', self.servers_settings_subpage.populate)

        # Long strip detection
        self.long_strip_detection_switch.set_active(self.settings.long_strip_detection)
        self.long_strip_detection_switch.connect('notify::active', self.on_long_strip_detection_changed)

        # NSFW content
        self.nsfw_content_switch.set_active(self.settings.nsfw_content)
        self.nsfw_content_switch.connect('notify::active', self.on_nsfw_content_changed)

        # NSFW only content
        self.nsfw_only_content_switch.set_active(self.settings.nsfw_only_content)
        self.nsfw_only_content_switch.connect('notify::active', self.on_nsfw_only_content_changed)

        #
        # Reader
        #

        # Reading mode
        self.reading_mode_row.set_selected(self.settings.reading_mode_value)
        self.reading_mode_row.connect('notify::selected', self.on_reading_mode_changed)

        # Image scaling filter
        self.scaling_filter_row.set_selected(self.settings.scaling_filter_value)
        self.scaling_filter_row.connect('notify::selected', self.on_scaling_filter_changed)

        # Background color
        self.background_color_row.set_selected(self.settings.background_color_value)
        self.background_color_row.connect('notify::selected', self.on_background_color_changed)

        # Page numbering
        self.page_numbering_switch.set_active(not self.settings.page_numbering)
        self.page_numbering_switch.connect('notify::active', self.on_page_numbering_changed)

        # Full screen
        self.fullscreen_switch.set_active(self.settings.fullscreen)
        self.fullscreen_switch.connect('notify::active', self.on_fullscreen_changed)

        #
        # Paged reading modes specific settings
        # Image scaling
        self.scaling_row.set_selected(self.settings.scaling_value)
        self.scaling_row.connect('notify::selected', self.on_scaling_changed)

        # Landscape pages zoom
        self.landscape_zoom_switch.set_active(self.settings.landscape_zoom)
        self.landscape_zoom_switch.connect('notify::active', self.on_landscape_zoom_changed)

        # Borders crop
        self.borders_crop_switch.set_active(self.settings.borders_crop)
        self.borders_crop_switch.connect('notify::active', self.on_borders_crop_changed)

        # Borders crop threshold
        model = Gtk.StringList()
        for threshold in BORDERS_CROP_THRESHOLDS:
            model.append(str(threshold))
        self.borders_crop_threshold_row.set_model(model)
        self.borders_crop_threshold_row.set_selected(BORDERS_CROP_THRESHOLDS.index(self.settings.borders_crop_threshold))
        self.borders_crop_threshold_row.connect('notify::selected', self.on_borders_crop_threshold_changed)

        #
        # Webtoon reading mode specific settings
        # Reset button
        self.webtoon_reading_mode_group_reset_button.connect('clicked', self.reset_webtoon_reading_mode_settings)

        # Pager clamp size
        self.clamp_size_adjustment.set_value(self.settings.clamp_size)
        self.clamp_size_adjustment.connect('value-changed', self.on_clamp_size_changed)

        # Scroll click percentage
        self.scroll_click_percentage_adjustment.set_value(self.settings.scroll_click_percentage)
        self.scroll_click_percentage_adjustment.connect('value-changed', self.on_scroll_click_percentage_changed)

        # Scroll drag factor
        self.scroll_drag_factor_adjustment.set_value(self.settings.scroll_drag_factor)
        self.scroll_drag_factor_adjustment.connect('value-changed', self.on_scroll_drag_factor_changed)

        # OCR language
        if self.window.reader.ocr_translator.enabled:
            model = Gtk.StringList()
            languages = self.window.reader.ocr_translator.ocr_languages
            for lang in languages:
                model.append(lang)
            self.ocr_lang_row.set_model(model)
            if self.settings.ocr_lang in languages:
                self.ocr_lang_row.set_selected(languages.index(self.settings.ocr_lang))
            self.ocr_lang_row.connect('notify::selected', self.on_ocr_lang_changed)
        else:
            self.ocr_lang_row.set_sensitive(False)

        #
        # Advanced
        #

        # Clear chapters cache and database
        self.clear_cached_data_button.connect('clicked', self.on_clear_cached_data_clicked)

        # Clear chapters cache and database on app close
        self.clear_cached_data_on_app_close_switch.set_active(self.settings.clear_cached_data_on_app_close)
        self.clear_cached_data_on_app_close_switch.connect('notify::active', self.on_clear_cached_data_on_app_close_changed)

        # Clear webview data
        self.clear_webview_data_button.connect('clicked', self.on_clear_webview_data_clicked)

        # Servers external modules
        self.servers_external_modules_switch.set_active(self.settings.servers_external_modules)
        self.servers_external_modules_switch.connect('notify::active', self.on_servers_external_modules_changed)

        # Servers bug report
        self.servers_bug_report_switch.set_active(self.settings.servers_bug_report)
        self.servers_bug_report_switch.connect('notify::active', self.on_servers_bug_report_changed)

        # Credentials storage: allow plaintext as fallback
        self.credentials_storage_plaintext_fallback_switch.set_active(self.settings.credentials_storage_plaintext_fallback)
        self.credentials_storage_plaintext_fallback_switch.connect('notify::active', self.on_credentials_storage_plaintext_fallback_changed)

        # Disable animations
        if Gtk.Settings.get_default().get_property('gtk-enable-animations'):
            Gtk.Settings.get_default().set_property('gtk-enable-animations', not Settings.get_default().disable_animations)
            self.disable_animations_switch.set_active(self.settings.disable_animations)
        else:
            # GTK animations are already disabled (in GNOME Settings for ex.)
            self.disable_animations_switch.get_parent().get_parent().get_parent().set_sensitive(False)
            self.disable_animations_switch.set_active(False)

        self.disable_animations_switch.connect('notify::active', self.on_disable_animations_changed)

    def show(self, transition=True):
        # Update maximum value of clamp size adjustment
        self.clamp_size_adjustment.set_upper(self.window.monitor.props.geometry.width)

        self.update_cached_data_size()
        self.update_webview_data_size()

        self.set_search_enabled(True)
        self.present(self.window)

    def update_cached_data_size(self):
        self.clear_cached_data_actionrow.set_subtitle(folder_size(get_cached_data_dir()) or '-')

    def update_webview_data_size(self):
        size = folder_size(get_webview_data_dir(), exclude='cookies.sqlite')
        self.clear_webview_data_actionrow.set_subtitle(size or '-')



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gettext import gettext as _
import logging
import re

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type
from komikku.utils import is_number

logger = logging.getLogger(__name__)


class Mangalivreblog(Server):
    id = 'mangalivreblog'
    name = 'Manga Livre'
    lang = 'pt_BR'

    base_url = 'https://mangalivre.blog'
    most_populars_url = base_url + '/wp-admin/admin-ajax.php'
    logo_url = base_url + '/wp-content/uploads/2025/05/cropped-logopp-32x32.png'
    search_url = base_url + '/pesquisa/'
    manga_url = base_url + '/manga/{0}/'
    chapter_url = base_url + '/capitulo/{0}/'

    filters = [
        {
            'key': 'status',
            'type': 'select',
            'name': _('Status'),
            'description': _('Filter by Status'),
            'value_type': 'single',
            'default': '',
            'options': [
                {'key': '', 'name': _('Any')},
                {'key': 'cancelado', 'name': 'Cancelado'},
                {'key': 'completo', 'name': 'Completo'},
                {'key': 'en-andamento', 'name': 'Em Andamento'},
                {'key': 'em-lancamento', 'name': 'Em Lançamento'},
                {'key': 'hiato', 'name': 'Hiato'},
            ],
        },
    ]
    long_strip_genres = ['Long Strip', 'Web Comic']

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers.update({'user-agent': USER_AGENT})

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],  # not available
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        data['name'] = soup.select_one('h1.manga-title').text.strip()
        data['cover'] = soup.select_one('.manga-cover img').get('src')

        # Details
        if element := soup.select_one('.manga-meta-item:-soup-contains("Status") .meta-value'):
            status = element.text.strip()
            if status in ('Em Andamento', 'Em Lançamento'):
                data['status'] = 'ongoing'
            elif status == 'Completo':
                data['status'] = 'complete'
            elif status == 'Cancelado':
                data['status'] = 'suspended'
            elif status in ('Hiato', 'Pausado'):
                data['status'] = 'hiatus'

        authors_selector = '.manga-meta-item:-soup-contains("Artista") .meta-value, .manga-meta-item:-soup-contains("Autor") .meta-value'
        for element in soup.select(authors_selector):
            authors = element.text.split(',')
            for author in authors:
                author = author.strip()
                if author not in data['authors']:
                    data['authors'].append(author)

        for element in soup.select('.manga-tag'):
            data['genres'].append(element.text.strip())

        # Synopsis
        data['synopsis'] = soup.select_one('.synopsis-content').text.strip()

        # Chapters
        for element in reversed(soup.select('.chapters-list .chapter-item')):
            a_element = element.select_one('.chapter-link')
            title_element = element.select_one('.chapter-number')
            date_element = element.select_one('.chapter-date')
            title = title_element.text.strip()
            num = title.split(' ')[-1]

            data['chapters'].append(dict(
                slug=a_element.get('href').split('/')[-2],
                title=title,
                num=num if is_number(num) else None,
                date=convert_date_string(date_element.text.strip(), languages=[self.lang]) if date_element else None,
            ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(self.chapter_url.format(chapter_slug))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = {
            'pages': [],
        }
        for element in soup.select('.chapter-image-container img'):
            data['pages'].append({
                'image': element.get('src'),
                'slug': None,
            })

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            page['image'],
            headers={
                'Referer': self.chapter_url.format(chapter_slug),
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_latest_updates(self, status=None):
        """
        Returns latest mangas
        """
        r = self.session_get(self.base_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select('section.latest-section .manga-card-modern'):
            a_element = element.select_one('.manga-title-modern a')
            img_element = element.select_one('.manga-cover-modern img')
            last_chapter_element = element.select_one('.chapter-list-modern .chapter-item-modern:first-child .chapter-number-modern')

            results.append({
                'slug': a_element.get('href').split('/')[-2],
                'name': a_element.text.strip(),
                'cover': img_element.get('src'),
                'last_chapter': last_chapter_element.text.strip() if last_chapter_element else None,
            })

        return results

    def get_most_populars(self, status=None):
        """
        Returns most popular mangas
        """
        r = self.session_get(self.base_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        nonce = None
        for script_element in soup.select('script'):
            script = script_element.string
            if not script or 'slimeReadPopular' not in script:
                continue

            if matches := re.search(r'nonce":"([^"]+)', script):
                nonce = matches.group(1)
            break

        if not nonce:
            return None

        r = self.session_post(
            self.most_populars_url,
            data={
                'action': 'get_popular_manga',
                'period': 'month',
                'nonce': nonce,
            },
            headers={
                'Referer': f'{self.base_url}/manga/',
                'X-Requested-With': 'XMLHttpRequest',
            }
        )
        if r.status_code != 200:
            return None

        resp_data = r.json()
        if 'success' not in resp_data or not resp_data['success']:
            return None

        soup = BeautifulSoup(resp_data['data']['html'], 'lxml')

        results = []
        for element in soup.select('.popular-manga-item'):
            a_element = element.select_one('.popular-manga-title a')
            img_element = element.select_one('.popular-manga-thumbnail img')

            results.append({
                'slug': a_element.get('href').split('/')[-2],
                'name': a_element.text.strip(),
                'cover': img_element.get('src'),
            })

        return results

    def search(self, term, status=None):
        params = {
            's': term,
        }
        if status:
            params['status'] = status

        r = self.session_get(self.base_url, params=params)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select('.manga-grid.search-results-grid .manga-card'):
            name_element = element.select_one('h3.manga-card-title')
            a_element = element.select_one('a.manga-card-link')
            img_element = element.img

            results.append({
                'slug': a_element.get('href').split('/')[-2],
                'name': name_element.text.strip(),
                'cover': img_element.get('src'),
            })

        return results



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gettext import gettext as _

from gi.repository import Adw
from gi.repository import GLib
from gi.repository import GObject
from gi.repository import Gtk

from komikku.models import Settings
from komikku.reader.pager import BasePager
from komikku.reader.pager.infinite_canvas import KInfiniteCanvas
from komikku.reader.pager.page import Page


class WebtoonPager(Adw.Bin, BasePager):
    """Vertical smooth/continuous scrolling (a.k.a. infinite canvas) pager"""

    __gtype_name__ = 'WebtoonPager'

    __interactive = True
    current_chapter_id = None
    scroll_page = None

    def __init__(self, reader):
        super().__init__()
        BasePager.__init__(self, reader)

        self.scrolledwindow = Gtk.ScrolledWindow()
        self.scrolledwindow.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        self.scrolledwindow.get_vscrollbar().set_visible(False)
        self.scrolledwindow.set_kinetic_scrolling(True)
        self.set_child(self.scrolledwindow)

        self.vadjustment_value_changed_handler_id = self.scrolledwindow.get_vadjustment().connect('value-changed', self.on_scroll)

        self.clamp = Adw.ClampScrollable()
        self.set_clamp_size()
        self.scrolledwindow.set_child(self.clamp)

        self.canvas = KInfiniteCanvas(self)
        self.canvas.connect('keyboard-navigation', self.on_keyboard_navigation)
        self.canvas.connect('controls-zone-clicked', self.on_controls_zone_clicked)
        self.canvas.connect('offlimit', self.on_offlimit)
        self.canvas.connect('page-requested', self.on_page_requested)
        self.clamp.set_child(self.canvas)

    @GObject.Property(type=bool, default=True)
    def interactive(self):
        return self.canvas.interactive

    @interactive.setter
    def interactive(self, value):
        self.canvas.interactive = value

    @property
    def pages(self):
        return self.canvas.pages

    def dispose(self):
        self.scrolledwindow.get_vadjustment().disconnect(self.vadjustment_value_changed_handler_id)

        self.canvas.dispose()
        self.canvas.unparent()
        self.canvas = None

    def goto_page(self, index):
        # TODO: use self.canvas.scroll_by_increment when possible
        self.canvas.disconnect_signals()
        self.canvas.clear()
        self.canvas.connect_signals()

        self.init(self.current_page.chapter, index)

    def init(self, chapter, page_index=None):
        if page_index is None:
            if chapter.read:
                page_index = 0
            elif chapter.last_page_read_index is not None:
                page_index = chapter.last_page_read_index
            else:
                page_index = 0

        page = Page(self, chapter, page_index)
        self.canvas.append(page)

        self.current_page = page

        GLib.idle_add(self.update, self.current_page)
        GLib.timeout_add(100, self.save_progress, page)

    def on_controls_zone_clicked(self, _canvas):
        self.reader.toggle_controls()

    def on_keyboard_navigation(self, _canvas):
        self.hide_cursor()

    def on_offlimit(self, _canvas, position):
        if position == 'bottom':
            message = _('It was the last chapter.')
        else:
            message = _('There is no previous chapter.')
        self.window.add_notification(message, timeout=1)

    def on_page_requested(self, _canvas, position):
        """
        Depending on the position parameter, adds a page at start (top) or at end (bottom)
        """

        if position == 'start':
            top_page = self.canvas.get_first_child()
            new_page = Page(self, top_page.chapter, top_page.index - 1)
            self.canvas.prepend(new_page)
        else:
            bottom_page = self.canvas.get_last_child()
            new_page = Page(self, bottom_page.chapter, bottom_page.index + 1)
            self.canvas.append(new_page)

    def on_scroll(self, _vadj):
        # Hide controls
        self.reader.toggle_controls(False)

        if self.canvas.scroll_direction == Gtk.DirectionType.DOWN:
            current_page = self.canvas.current_page_bottom
        else:
            current_page = self.canvas.current_page_top

        if current_page and current_page != self.current_page:
            self.current_page = current_page

            GLib.idle_add(self.update, current_page)

            pages = self.pages
            GLib.timeout_add(
                100, self.save_progress,
                pages[pages.index(self.canvas.current_page_top):pages.index(self.current_page) + 1]
            )

    def set_clamp_size(self, zoom=1):
        """
        Adjust clamp width (increase/decrease) by a zoom factor

        Use to allow pseudo-zoom limited to the width of the window

        :param zoom: A zoom factor to be applied to the width of the clamp
        :type zoom: float

        :return: Zoom factor applied taking into account size limitations
        :rtype: float
        """
        size = min(max(360, Settings.get_default().clamp_size * zoom), self.reader.window.monitor.props.geometry.width)

        self.clamp.set_maximum_size(size)
        self.clamp.set_tightening_threshold(size)

        return size / Settings.get_default().clamp_size

    def set_orientation(self, _orientation):
        return

    def update(self, page, _direction=None):
        if self.window.page != 'reader' or page.status == 'disposed' or self.current_page != page:
            return GLib.SOURCE_REMOVE

        if not page.loadable and page.error is None:
            # Loop until page is loadable or page is on error
            return GLib.SOURCE_CONTINUE

        # Update title, initialize controls and notify user if chapter changed
        if self.current_chapter_id != page.chapter.id:
            self.current_chapter_id = page.chapter.id

            self.reader.update_title(page.chapter)
            self.window.add_notification(page.chapter.title, timeout=2)
            self.reader.controls.init(page.chapter)

        if not page.loadable:
            self.window.add_notification(_('This chapter is inaccessible.'), timeout=2)

        # Update page number and controls page slider
        self.reader.update_page_numbering(page.index + 1, len(page.chapter.pages) if page.loadable else None)
        self.reader.controls.set_scale_value(page.index + 1)

        return GLib.SOURCE_REMOVE



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

# My Manga Reader CMS

# Supported servers:
# Bentoscan [FR] (Disabled)
# FR Scan [FR] (Disabled)
# Jpmangas [FR] (Disabled)
# Mangadoor [ES] (Disabled)
# Manga Scan [FR] (Disabled)
# Mangasin [ES]
# Scan FR [FR] (Disabled)
# Scan OP [FR] (Disabled)
# ScanOnePiece [FR]

import re
from urllib.parse import urljoin

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.servers.utils import get_soup_element_inner_text
from komikku.utils import get_buffer_mime_type


class MyMangaReaderCMS(Server):
    base_url: str
    search_url: str
    most_populars_url: str
    manga_url: str
    chapter_url: str
    image_url: str
    cover_url: str

    search_query_param: str = 'query'

    details_name_selector: str = None

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers.update({'user-agent': USER_AGENT})

    @staticmethod
    def extract_chapter_nums_from_slug(slug):
        """Extract chapter num from slug

        Volume num is not available
        """
        re_nums = r'^((\w+)-)?(\d+)(\.(\d+))?.*'

        if matches := re.search(re_nums, slug):
            if num := matches.group(3):
                num = f'{int(num)}'

                if num_dec := matches.group(5):
                    num = f'{num}.{int(num_dec)}'

                return num, None

        return None, None

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r is None:
            return None

        mime_type = get_buffer_mime_type(r.content)

        if r.status_code != 200 or mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        if self.details_name_selector:
            data['name'] = soup.select_one(self.details_name_selector).text.strip()
        else:
            data['name'] = soup.find('img', class_=['lazyload', 'img-responsive']).get('alt').strip()
        data['cover'] = self.cover_url.format(data['slug'])

        # Details
        elements = soup.find('dl', class_='dl-horizontal').findChildren(recursive=False)
        for element in elements:
            if element.name not in ('dt', 'dd'):
                continue

            if element.name == 'dt':
                label = element.text.strip()
                continue

            if label.startswith(('Author', 'Auteur', 'Autor', 'Artist')):
                value = element.text.strip()
                for t in value.split(','):
                    t = t.strip()
                    if t not in data['authors']:
                        data['authors'].append(t)
            elif label.startswith(('Categories', 'Catégories', 'Categorías', 'Genres', 'Género', 'Tags')):
                data['genres'] = [a_element.text.strip() for a_element in element.find_all('a')]
            elif label.startswith(('Status', 'Statut', 'Estado')):
                value = element.text.strip().lower()
                if value in ('ongoing', 'en cours', 'en curso'):
                    data['status'] = 'ongoing'
                elif value in ('complete', 'terminé', 'completa'):
                    data['status'] = 'complete'

        if synopsis_element := soup.select_one('.well'):
            if synopsis_element.h5:
                synopsis_element.h5.extract()

            data['synopsis'] = synopsis_element.text.strip()
            if alert_element := soup.select_one('.alert-danger'):
                data['synopsis'] += '\n\n' + alert_element.text.strip()

        data['chapters'] = self.get_manga_chapters_data(soup)

        return data

    def get_manga_chapters_data(self, soup):
        chapters = []
        elements = soup.find('ul', class_=re.compile(r'chapter.*')).find_all('li')
        for element in reversed(elements):
            h5 = element.h5
            if not h5:
                continue

            slug = h5.a.get('href').split('/')[-1]
            num, _num_volume = self.extract_chapter_nums_from_slug(slug)
            title = h5.a.text.strip()
            if h5.em and h5.em.text.strip():
                title = '{0}: {1}'.format(title, h5.em.text.strip())
            date = element.div.div.text.strip()

            chapters.append(dict(
                slug=slug,
                title=title,
                num=num,
                date=convert_date_string(date, format='%d %b. %Y'),
            ))

        return chapters

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages (list of images filenames) are expected.
        """
        r = self.session_get(self.chapter_url.format(manga_slug, chapter_slug))
        if r is None:
            return None

        mime_type = get_buffer_mime_type(r.content)

        if r.status_code != 200 or mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        pages_imgs = soup.find('div', id='all').find_all('img')

        data = dict(
            pages=[],
        )
        for index, img in enumerate(pages_imgs):
            if self.image_url:
                slug = img.get('data-src').strip().split('/')[-1]
                image = None
            else:
                slug = None
                src = img.get('data-src').strip()
                if src.startswith('http'):
                    image = src
                elif src.startswith('//'):
                    image = 'https:' + src
                else:
                    image = urljoin(self.base_url, src.lstrip('/'))

            data['pages'].append(dict(
                slug=slug,
                image=image,
                index=index + 1,
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        headers = {
            'Referer': self.chapter_url.format(manga_slug, chapter_slug),
        }
        if page['slug']:
            r = self.session_get(self.image_url.format(manga_slug, chapter_slug, page['slug']), headers=headers)
        else:
            r = self.session_get(page['image'], headers=headers)
        if r is None or r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['slug'] if page['slug'] else '{0}.{1}'.format(page['index'], mime_type.split('/')[-1]),
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_latest_updates(self):
        """
        Returns list of latest updated manga
        """
        r = self.session_get(
            f'{self.base_url}/latest-release'
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select('.mangalist .manga-item'):
            slug = element.a.get('href').split('/')[-1]

            results.append(dict(
                name=element.a.text.strip(),
                slug=slug,
                cover=self.cover_url.format(slug),
            ))

        return results

    def get_most_populars(self):
        """
        Returns list of most viewed manga
        """
        r = self.session_get(
            self.most_populars_url,
            headers={
                'Referer': self.base_url + '/manga-list',
                'x-requested-with': 'XMLHttpRequest'
            }
        )
        if r is None:
            return None

        mime_type = get_buffer_mime_type(r.content)

        if r.status_code != 200 or mime_type not in ('text/html', 'text/plain'):
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for a_element in soup.find_all('a', class_='chart-title'):
            name = a_element.get('title')
            if not name:
                name = a_element.text
            slug = a_element.get('href').split('/')[-1]

            results.append(dict(
                name=name.strip(),
                slug=slug,
                cover=self.cover_url.format(slug),
            ))

        return results

    def search(self, term):
        params = {}
        params[self.search_query_param] = term
        r = self.session_get(
            self.search_url,
            params=params
        )
        if r is None:
            return None

        if r.status_code == 200:
            try:
                # Returned data for each manga:
                # value: name of the manga
                # data: slug of the manga
                data = r.json()
                if 'suggestions' in data:
                    data = data['suggestions']

                results = []
                for item in data:
                    results.append(dict(
                        slug=item['data'],
                        name=item['value'],
                        cover=self.cover_url.format(item['data']),
                    ))
            except Exception:
                return None
            else:
                return results

        return None


class MyMangaReaderCMSv1(MyMangaReaderCMS):
    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r is None:
            return None

        mime_type = get_buffer_mime_type(r.content)

        if r.status_code != 200 or mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        data['name'] = get_soup_element_inner_text(soup.find('div', class_='panel-heading').h3)
        data['cover'] = self.cover_url.format(data['slug'])

        # Details
        elements = soup.find('div', class_=['panel', 'panel-default']).find_all('span', class_='list-group-item')
        for element in elements:
            label = element.b.text.strip()
            element.b.extract()
            value = element.text.strip()

            if label.startswith(('Author', 'Auteur', 'Autor', 'Artist')):
                for author in value.split(','):
                    author = author.strip()
                    if author not in data['authors']:
                        data['authors'].append(author)
            elif label.startswith(('Categories', 'Catégories', 'Categorías', 'Género')):
                data['genres'] = [genre.strip() for genre in value.split(',')]
            elif label.startswith(('Status', 'Statut', 'Estado')):
                value = value.lower()
                if value in ('ongoing', 'en cours'):
                    data['status'] = 'ongoing'
                elif value in ('complete', 'terminé'):
                    data['status'] = 'complete'
            elif label.startswith('Resumen'):
                data['synopsis'] = value

        # Chapters
        elements = soup.find('div', class_='capitulos-list').find_all('tr')
        for element in reversed(elements):
            td_elements = element.find_all('td')

            data['chapters'].append(dict(
                slug=td_elements[0].a.get('href').split('/')[-1],
                date=convert_date_string(td_elements[1].text.strip(), format='%d %b. %Y'),
                title=td_elements[0].a.text.strip(),
            ))

        return data

    def get_most_populars(self):
        """
        Returns list of most viewed manga
        """
        r = self.session_get(self.most_populars_url)
        if r is None:
            return None

        mime_type = get_buffer_mime_type(r.content)

        if r.status_code != 200 or mime_type not in ('text/html', 'text/plain'):
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.find_all('div', class_='thumbnail'):
            results.append(dict(
                name=element.a.img.get('alt').strip(),
                slug=element.a.get('href').split('/')[-1],
            ))

        return results



# SPDX-FileCopyrightText: 2020-2024 TAAPArthur
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: TAAPArthur

import logging

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.servers.utils import do_login
from komikku.utils import get_buffer_mime_type

SERVER_NAME = 'Crunchyroll'

logger = logging.getLogger('komikku.servers.crunchyroll')


class Crunchyroll(Server):
    id = 'crunchyroll'
    name = SERVER_NAME
    lang = 'en'
    locale = 'enUS'
    has_login = True
    status = 'disabled'

    base_url = 'https://www.crunchyroll.com'
    manga_url = base_url + '/comics/manga/{0}/volumes'

    start_session_url = 'https://api.crunchyroll.com/start_session.0.json'
    login_url = 'https://api.crunchyroll.com/login.0.json'

    api_base_url = 'https://api-manga.crunchyroll.com'
    api_auth_url = api_base_url + '/cr_authenticate?auth=&session_id={}&version=0&format=json'
    api_series_url = api_base_url + '/series?sort=popular'
    api_chapter_url = api_base_url + '/list_chapter?session_id={}&chapter_id={}&auth={}'
    api_chapters_url = api_base_url + '/chapters?series_id={}'

    api_auth_token = None
    api_session_id = None
    possible_page_url_keys = ['encrypted_mobile_image_url', 'encrypted_composed_image_url']
    page_url_key = possible_page_url_keys[0]

    # _access_token = 'WveH9VkPLrXvuNm'
    # _access_type = 'com.crunchyroll.crunchyroid'
    _access_token = 'giKq5eY27ny3cqz'
    _access_type = 'com.crunchyroll.static'

    headers = {
        'User-Agent': USER_AGENT,
        'Origin': 'https://www.crunchyroll.com',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Language': 'en,en-US;q=0.9',
        'Accept-Encoding': 'gzip, deflate, br',
        'DNT': '1',
        'Connection': 'keep-alive',
    }

    def __init__(self, username=None, password=None):
        if username and password:
            self.do_login(username, password)

    def _get_session_id(self):
        if 'session_id' in self.session.cookies:
            self.api_session_id = self.session.cookies['session_id']
            return

        data = self.session_post(
            self.start_session_url,
            data={
                'device_id': '1234567',
                'device_type': self._access_type,
                'access_token': self._access_token,
            }
        ).json()['data']
        self.api_session_id = data['session_id']

    @staticmethod
    def decode_image(buffer):
        # Don't know why 66 is special
        return bytes(b ^ 66 for b in buffer)

    @do_login
    def get_manga_data(self, initial_data):
        """
        Returns manga data from API

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'
        r = self.session_get(self.api_chapters_url.format(initial_data['slug']))

        json_data = r.json()
        if 'error' in json_data:
            logger.warning('Manga slug {0}: {1}'.format(initial_data['slug'], json_data['error']['msg']))
            return None

        resp_data = json_data['series']
        chapters = json_data['chapters']

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status='ongoing',
            chapters=[],
            synopsis=resp_data['locale'][self.locale]['description'],
            server_id=self.id,
            cover=resp_data['locale'][self.locale]['thumb_url'],
            url=self.manga_url.format(resp_data['url'][1:]),
        ))

        if resp_data.get('authors'):
            data['authors'] += [t.strip() for t in resp_data['authors'].split(',')]
        if resp_data.get('artist'):
            data['authors'] += [t.strip() for t in resp_data['artist'].split(',') if t.strip() not in data['authors']]

        if resp_data.get('translator'):
            data['scanlators'] += [t.strip() for t in resp_data['translator'].split('|')]

        if resp_data.get('genres'):
            data['genres'] = resp_data['genres']

        if resp_data['locale'][self.locale].get('copyright'):
            data['synopsis'] += '\n\n' + resp_data['locale'][self.locale]['copyright']

        # Chapters
        for chapter in chapters:
            date = None
            if chapter.get('availability_start'):
                date_string = chapter['availability_start'].split(' ')[0]
                if len(date_string) == 10 and '-00' not in date_string:
                    date = convert_date_string(date_string, '%Y-%m-%d')
            if date is None and chapter.get('updated'):
                date_string = chapter['updated'].split(' ')[0]
                if len(date_string) == 10 and '-00' not in date_string:
                    date = convert_date_string(date_string, '%Y-%m-%d')

            data['chapters'].append(dict(
                slug=chapter['chapter_id'],
                title=chapter['locale'][self.locale]['name'],
                date=date,
            ))

        return data

    @do_login
    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data

        Currently, only pages are expected.
        """
        if self.logged_in and (self.api_session_id is None or self.api_auth_token is None):
            if not self.init_api():
                return None

        r = self.session_get(self.api_chapter_url.format(self.api_session_id, chapter_slug, self.api_auth_token))

        resp_data = r.json()
        if resp_data is None or resp_data.get('error', False):
            # We aren't logged in
            return None

        pages = resp_data['pages']
        pages.sort(key=lambda x: int(x['number']))

        data = dict(
            pages=[],
        )

        for page in pages:
            data['pages'].append(dict(
                slug=None,
                image=page['locale'][self.locale][self.page_url_key],
            ))

        return data

    @do_login
    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(page['image'])
        if r.status_code != 200:
            return None

        buffer = self.decode_image(r.content)

        mime_type = get_buffer_mime_type(buffer)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=buffer,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    @staticmethod
    def get_manga_url(slug, url):
        """
        Returns manga absolute URL
        """
        return url

    @do_login
    def get_most_populars(self):
        """
        Returns full list of manga sorted by rank
        """
        r = self.session_get(self.api_series_url)

        result = []

        resp_data = r.json()
        resp_data.sort(key=lambda x: not x['featured'])

        for item in resp_data:
            if 'locale' not in item:
                continue

            result.append({
                'slug': item['series_id'],
                'name': item['locale'][self.locale]['name'],
            })

        return result

    def init_api(self):
        """
        Retrieves API session ID and authentication token
        """
        self._get_session_id()
        r = self.session_get(self.api_auth_url.format(self.api_session_id))
        data = r.json()

        if 'data' in data:
            self.api_auth_token = ''.join(data['data']['auth'])
            return True

        return False

    def login(self, username, password):
        """
        Log in and initializes API
        """
        if not username or not password:
            return False
        self._get_session_id()

        login = self.session_post(
            self.login_url,
            data={
                'session_id': self.api_session_id,
                'account': username,
                'password': password
            }
        ).json()

        if 'data' in login:
            self.api_auth_token = login['data']['auth']
            self.save_session()
            return True

        return False

    def search(self, term):
        return list(filter(lambda x: term.lower() in x['name'].lower(), self.get_most_populars()))



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gi.repository import Adw
from gi.repository import Gdk
from gi.repository import GLib
from gi.repository import GObject
from gi.repository import Graphene
from gi.repository import Gsk
from gi.repository import Gtk


class SynopsisFading(Gtk.Widget):
    __gtype_name__ = 'SynopsisFading'

    def __init__(self):
        super().__init__()

        self.__max_height = 95
        self.__faded = False

        self.min_height = self.__max_height
        self.current_height = 0
        self.allocated_width = 0

        self.animation = Adw.SpringAnimation(
            widget=self,
            value_from=0,
            value_to=1,
            spring_params=Adw.SpringParams.new(damping_ratio=0.5, mass=1, stiffness=50),
            target=Adw.CallbackAnimationTarget.new(self.animate),
            initial_velocity=1,
            epsilon=0.001,
            clamp=True
        )
        self.child = None

    @GObject.Property(type=bool, default=False)
    def faded(self):
        return self.__faded

    @faded.setter
    def faded(self, value):
        self.__faded = value

    @GObject.Property(type=int, default=95)
    def max_height(self):
        return self.__max_height

    @max_height.setter
    def max_height(self, value):
        if self.max_height == value:
            return

        self.__max_height = value

        if self.child:
            if self.allocated_width > 0:
                width = self.allocated_width
            else:
                _, width, _, _ = self.child.measure(Gtk.Orientation.HORIZONTAL, -1)

            _, child_height, _, _ = self.child.measure(Gtk.Orientation.VERTICAL, width)
            if child_height <= value:
                target_height = child_height
            else:
                target_height = value
        else:
            target_height = value

        self.animation_height_start = self.current_height
        self.animation_height_end = target_height
        self.animation.play()

    def animate(self, value):
        self.current_height = Adw.lerp(self.animation_height_start, self.animation_height_end, value)
        self.queue_resize()

    def do_measure(self, orientation, for_size):
        if not self.child:
            return 0, 0, -1, -1

        if orientation == Gtk.Orientation.HORIZONTAL:
            return self.child.measure(orientation, for_size)

        if for_size == -1:
            if self.allocated_width > 0:
                for_size = self.allocated_width
            else:
                _, for_size, _, _ = self.child.measure(Gtk.Orientation.HORIZONTAL, -1)

        _, child_height, _, _ = self.child.measure(Gtk.Orientation.VERTICAL, for_size)
        if child_height <= self.max_height:
            target_height = child_height
        else:
            target_height = self.max_height

        if self.animation.get_state() == Adw.AnimationState.IDLE:
            self.current_height = target_height

        return self.current_height, self.current_height, -1, -1

    def do_size_allocate(self, width, height, baseline):
        if self.allocated_width != width:
            self.allocated_width = width
            if self.child:
                _, child_height, _, _ = self.child.measure(Gtk.Orientation.VERTICAL, width)

                if self.min_height != self.max_height or child_height < self.min_height:
                    self.current_height = child_height
                GLib.idle_add(self.queue_resize)

        if self.child:
            _, child_height, _, _ = self.child.measure(Gtk.Orientation.VERTICAL, width)
            if child_height > height:
                child_height = child_height
            else:
                child_height = height

            self.child.allocate(width, child_height, baseline, None)

        self.update_faded()

    def do_snapshot(self, snapshot):
        if not self.child:
            return

        width = self.get_width()
        height = self.get_height()
        if height <= 0:
            return

        _, child_height, _, _ = self.child.measure(Gtk.Orientation.VERTICAL, width)
        if child_height <= height:
            self.snapshot_child(self.child, snapshot)
            return

        # Fading
        snapshot.push_mask(Gsk.MaskMode.ALPHA)

        stop_offset = Adw.lerp(0.25, 1, (height - self.min_height) / (child_height - self.min_height))

        cs0 = Gsk.ColorStop()
        cs0.color = Gdk.RGBA(1, 1, 1, 1)
        cs0.offset = stop_offset
        cs1 = Gsk.ColorStop()
        cs1.color = Gdk.RGBA(1, 1, 1, 0)
        cs1.offset = 1

        snapshot.append_linear_gradient(
            Graphene.Rect.init(Graphene.Rect.alloc(), 0, 0, width, height),
            Graphene.Point.init(Graphene.Point.alloc(), 0, 0),
            Graphene.Point.init(Graphene.Point.alloc(), 0, height),
            [cs0, cs1]
        )

        snapshot.pop()

        self.snapshot_child(self.child, snapshot)

        snapshot.pop()

    def get_request_mode(self):
        return Gtk.SizeRequestMode.HEIGHT_FOR_WIDTH

    def set_child(self, child):
        if self.child:
            self.child.unparent()

        self.child = child
        self.child.set_parent(self)

    def set_markup(self, text):
        self.__faded = None
        self.__max_height = self.min_height
        self.current_height = self.min_height
        self.allocated_width = 0
        self.child.set_markup(text)

    def set_revealed(self, revealed):
        self.max_height = self.child.get_height() if revealed else self.min_height

    def update_faded(self):
        new_value = False

        if self.child:
            if self.allocated_width > 0:
                width = self.allocated_width
            else:
                _, width, _, _ = self.child.measure(Gtk.Orientation.HORIZONTAL, -1)
            _, child_height, _, _ = self.child.measure(Gtk.Orientation.VERTICAL, width)

            new_value = child_height > self.min_height

        if self.faded != new_value:
            self.faded = new_value



# SPDX-FileCopyrightText: 2024-2024 Zhao Se
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Zhao Se <zhaose233@outlook.com>

import json

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type


class Rawdevart(Server):
    id = 'rawdevart'
    name = 'Rawdevart'
    lang = 'ja'

    is_nsfw = True

    base_url = 'https://rawdevart.art'
    logo_url = base_url + '/logo.png'
    media_url = 'https://s3.rawuwu.com'
    manga_url = base_url + '/{0}'
    api_search_url = base_url + '/ajax/search-manga'
    latest_updates_url = base_url + '/latest'
    api_latest_updates_url = base_url + '/spa/latest-manga'
    api_manga_url = base_url + '/spa/manga/{0}'
    api_chapter_url = api_manga_url + '/{1}'

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers.update({'User-Agent': USER_AGENT})

    def get_manga_data(self, initial_data):
        """
        Get full manga data by parsing the response of the API.
        """
        r = self.session_get(self.api_manga_url.format(initial_data['slug']))
        if r.status_code != 200:
            return None

        r_decoded = json.loads(r.text)

        authors = [
            author_info['author_name'] for author_info in r_decoded['authors']
        ]
        genres = [tag_info['tag_name'] for tag_info in r_decoded['tags']]
        synopsis = r_decoded['detail']['manga_description']

        chapters = []
        for chapter_info in reversed(r_decoded['chapters']):
            slug = str(chapter_info['chapter_number'])
            title = f'Chapter {slug}'
            date = convert_date_string(chapter_info['chapter_date_published'].split('T')[0])

            chapters.append(dict(slug=slug, title=title, num=slug, date=date))

        # Slug can't be used to compute manga URL
        # Manga URL must be computed and recorded
        # It's computed using part of cover URL
        manga_url_slug_split = r_decoded['detail']['manga_cover_img_full'].split('/')[-1].split('-')[:-1]
        manga_url_slug_split[-1] = 'c' + manga_url_slug_split[-1]
        manga_url_slug = '-'.join(manga_url_slug_split)

        return dict(
            name=r_decoded['detail']['manga_name'].strip(),
            slug=initial_data['slug'],
            url=self.manga_url.format(manga_url_slug),
            authors=authors,
            genres=genres,
            status=None,  # not available
            synopsis=synopsis,
            chapters=chapters,
            server_id=self.id,
            cover=r_decoded['detail']['manga_cover_img_full'],
        )

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by parsing the response of the API.
        """
        r = self.session_get(self.api_chapter_url.format(manga_slug, chapter_slug))
        if r.status_code != 200:
            return None

        r_decoded = json.loads(r.text)

        soup = BeautifulSoup(r_decoded['chapter_detail']['chapter_content'], 'lxml')

        pages = []
        for index, canvas in enumerate(soup.find_all('canvas')):
            image = canvas.get('data-srcset')
            pages.append(dict(
                image=image,
                server=r_decoded['chapter_detail']['server'],
                index=index + 1,
            ))

        return dict(pages=pages)

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            f'{page.get("server") or self.media_url}/{page["image"]}',
            headers={
                'Referer': f'{self.base_url}/',
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name='{0:03d}.{1}'.format(page['index'], mime_type.split('/')[-1]),
        )

    def get_manga_url(self, slug, url):
        return url  # It's hard to get the absolute webpage url

    def get_latest_updates(self):
        """
        Returns latest updates
        """
        r = self.session_get(
            self.api_latest_updates_url,
            headers={
                'Referer': self.latest_updates_url,
            }
        )
        if r.status_code != 200:
            return None

        r_decoded = json.loads(r.text)

        results = []
        for manga_info in r_decoded['manga_list']:
            slug = manga_info['manga_id']
            name = manga_info['manga_name']
            cover = manga_info['manga_cover_img']
            if chapters := manga_info.get('manga_chapters'):
                last_chapter = 'Chapter ' + str(chapters[0]['chapter_number'])
            else:
                last_chapter = None

            results.append(dict(
                slug=str(slug),
                name=name,
                cover=cover,
                last_chapter=last_chapter,
            ))

        return results

    def search(self, term=None):
        """
        Search for mangas
        """
        r = self.session_get(self.api_search_url, params=dict(query=term))
        if r.status_code != 200:
            return None

        r_decoded = json.loads(r.text)

        results = []
        for manga_info in r_decoded:
            slug = manga_info['manga_id']
            name = manga_info['manga_name']
            cover = manga_info['manga_cover_img']
            nb_chapters = manga_info['chapter_number']

            results.append(dict(
                slug=str(slug),
                name=name,
                cover=cover,
                nb_chapters=nb_chapters,
            ))

        return results



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from bs4 import BeautifulSoup

from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type
from komikku.webview import CompleteChallenge


class Goldenmangas(Server):
    id = 'goldenmangas'
    name = 'Golden Mangás'
    lang = 'pt_BR'
    is_nsfw = True
    status = 'disabled'

    has_cf = True

    base_url = 'https://goldenmanga.top'
    search_url = base_url + '/mangas'
    manga_url = base_url + '/mangas/{0}'
    chapter_url = base_url + '/mangas/{0}/{1}'
    image_url = base_url + '/mm-admin/uploads/mangas/{0}/{1}/{2}'

    @CompleteChallenge()
    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        info_element = soup.select_one('div.container.manga > div.row')

        # Name & cover
        data['name'] = info_element.select_one('div.col-sm-8 > div.row > div.col-sm-8 > h2:nth-child(1)').text.strip()
        data['cover'] = '{0}{1}'.format(
            self.base_url,
            info_element.select_one('div.col-sm-8 > div.row > div.col-sm-4.text-right > img').get('src')
        )

        # Details
        for h5_element in info_element.select('div.col-sm-8 > div.row > div.col-sm-8 > h5'):
            label = h5_element.strong.text.strip().lower()

            if label.startswith('genero'):
                for a_element in h5_element.find_all('a'):
                    if genre := a_element.text.strip():
                        data['genres'].append(genre)

            elif label.startswith(('autor', 'artista')):
                for a_element in h5_element.find_all('a'):
                    author = a_element.text.strip()
                    if author and author not in data['authors']:
                        data['authors'].append(author)

            elif label.startswith('status'):
                status = h5_element.a.text.strip().lower()
                if status == 'completo':
                    data['status'] = 'complete'
                elif status == 'ativo':
                    data['status'] = 'ongoing'

        # Synopsis
        data['synopsis'] = info_element.find(id='manga_capitulo_descricao').text.strip()

        # Chapters
        for li_element in reversed(soup.find(id='capitulos').find_all('li')):
            title_element = li_element.select_one('a > div.col-sm-5')
            date_element = title_element.span.extract()
            scanlators = None
            if scanlator_elements := li_element.select('div > a.label.label-default'):
                scanlators = [scanlator_element.text.strip() for scanlator_element in scanlator_elements]

            data['chapters'].append(dict(
                slug=li_element.a.get('href').split('/')[-1],
                title=title_element.text.strip(),
                scanlators=scanlators,
                date=convert_date_string(date_element.text.strip()[1:-1], format='%d/%m/%Y'),
            ))

        return data

    @CompleteChallenge()
    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(self.chapter_url.format(manga_slug, chapter_slug))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = dict(
            pages=[],
        )
        for img_element in soup.find(id='capitulos_images').find_all('img', class_='img-manga'):
            data['pages'].append(dict(
                slug=img_element.get('src').split('/')[-1],
                image=None,
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(self.image_url.format(manga_slug, chapter_slug, page['slug']))
        if r.status_code != 200:
            return None

        buffer = r.content
        mime_type = get_buffer_mime_type(buffer)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=buffer,
            mime_type=mime_type,
            name=page['slug'],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    @CompleteChallenge()
    def get_latest_updates(self):
        r = self.session_get(self.base_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text.encode('utf-8'), 'lxml', from_encoding='utf-8')

        results = []
        for a_element in soup.select('.atualizacao > a'):
            results.append(dict(
                name=a_element.find('h3').text.strip(),
                slug=a_element.get('href').split('/')[-1],
            ))

        return results

    @CompleteChallenge()
    def get_most_populars(self, types=None, statuses=None):
        r = self.session_get(self.base_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text.encode('utf-8'), 'lxml', from_encoding='utf-8')

        results = []
        for a_element in soup.select('#capitulosdestaque > a'):
            results.append(dict(
                name=a_element.find_all('span')[-1].text.strip(),
                slug=a_element.get('href').split('/')[-2],
            ))

        return results

    @CompleteChallenge()
    def search(self, term, types=None, statuses=None, orderby=None):
        r = self.session_get(self.search_url, params=dict(busca=term))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.find_all(class_='mangas'):
            results.append(dict(
                name=element.h3.text.strip(),
                slug=element.a.get('href').split('/')[-1],
            ))

        return results



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gettext import gettext as _
import json
import time

from bs4 import BeautifulSoup
import requests

from komikku.consts import DOWNLOAD_MAX_DELAY
from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type
from komikku.utils import get_response_elapsed

SEARCH_RESULTS_PAGES = 4
SEARCH_RESULTS_PAGE_LIMIT = 24


class Mangak(Server):
    id = 'mangak'
    name = 'MangaK'
    lang = 'en'

    is_nsfw = True

    base_url = 'https://mangak.io'
    logo_url = base_url + '/static/sites/mangak/icons/favicon-32x32.png'

    search_url = base_url + '/search'
    manga_url = base_url + '/{0}'
    chapter_url = base_url + '/{0}/{1}'

    api_url = 'https://api.mangak.io'
    api_search_url = api_url + '/titles/search'
    api_chapters_url = api_url + '/titles/{0}/chapters'

    filters = [
        {
            'key': 'content_rating',
            'type': 'select',
            'name': _('Content Rating'),
            'description': _('Filter by Content Rating'),
            'value_type': 'single',
            'default': '',
            'options': [
                {'key': '', 'name': _('Any')},
                {'key': 'safe', 'name': _('Safe')},
                {'key': 'suggestive', 'name': _('Suggestive')},
                {'key': 'erotica', 'name': _('Erotica')},
                {'key': 'pornographic', 'name': _('Pornographic')},
            ]
        },
        {
            'key': 'type',
            'type': 'select',
            'name': _('Type'),
            'description': _('Filter by Type'),
            'value_type': 'single',
            'default': '',
            'options': [
                {'key': '', 'name': _('Any')},
                {'key': 'manga', 'name': _('Manga')},
                {'key': 'manhwa', 'name': _('Manhwa')},
                {'key': 'manhua', 'name': _('Manhua')},
            ]
        },
        {
            'key': 'demographic',
            'type': 'select',
            'name': _('Demographic'),
            'description': _('Filter by Publication Demographic'),
            'value_type': 'single',
            'default': '',
            'options': [
                {'key': '', 'name': _('Any')},
                {'key': 'shounen,seinen', 'name': _('Boy (Shounen + Seinen)')},
                {'key': 'shoujo,josei', 'name': _('Girl (Shoujo + Josei)')},
                {'key': 'shounen', 'name': _('Shounen')},
                {'key': 'shoujo', 'name': _('Shoujo')},
                {'key': 'seinen', 'name': _('Seinen')},
                {'key': 'josei', 'name': _('Josei')},
            ]
        },
        {
            'key': 'status',
            'type': 'select',
            'name': _('Status'),
            'description': _('Filter by Status'),
            'value_type': 'single',
            'default': '',
            'options': [
                {'key': '', 'name': _('Any')},
                {'key': 'ongoing', 'name': _('Ongoing')},
                {'key': 'completed', 'name': _('Completed')},
                {'key': 'hiatus', 'name': _('Hiatus')},
                {'key': 'cancelled', 'name': _('Canceled')},
            ]
        },
    ]

    headers = {
        'User-Agent': USER_AGENT,
    }

    long_strip_genres = [
        'Webtoons',
    ]

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers = self.headers

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Slug is missing in initial data'

        id_, slug = initial_data['slug'].split(':')

        r = self.session_get(self.manga_url.format(slug))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        props = None
        if script_element := soup.select_one('script#__NEXT_DATA__'):
            script = script_element.string
            if script:
                props = json.loads(script)['props']['pageProps']['initialManga']

        if props is None:
            return None

        data = initial_data.copy()
        data.update(dict(
            name=props['name'],
            authors=[],
            scanlators=[],  # not available?
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=props.get('cover'),
        ))

        # Details
        if authors := props.get('authors'):
            for author in authors:
                data['authors'].append(author['name'])
        if artists := props.get('artists'):
            for artist in artists:
                artist = artist['name']
                if artist not in data['authors']:
                    data['authors'].append(artist)

        if props['status'] == 'Completed':
            data['status'] = 'complete'
        elif props['status'] == 'Ongoing':
            data['status'] = 'ongoing'
        elif props['status'] == 'Hiatus':
            data['status'] = 'hiatus'
        elif props['status'] == 'Cancelled':
            data['status'] = 'suspended'

        if genres := props.get('genres'):
            for genre in genres:
                data['genres'].append(genre['name'])
        if demographics := props.get('demographics'):
            for demographic in demographics:
                demographic = demographic['name']
                if demographic not in data['genres']:
                    data['genres'].append(demographic)
        if type_ := props.get('type'):
            type_ = type_['name']
            if type_ not in data['genres']:
                data['genres'].append(type_)
        if content_rating := props.get('contentRating'):
            data['genres'].append(content_rating)

        if synopsis := props.get('summary'):
            data['synopsis'] = synopsis

        # Chapters
        r = self.session_get(
            self.api_chapters_url.format(id_),
            params={
                'cv': int(time.time() * 1000),
            },
            headers={
                'Referer': self.manga_url.format(slug),
            }
        )
        if r.status_code != 200:
            return None

        resp_data = r.json()
        if not resp_data.get('success'):
            return None

        for chapter in reversed(resp_data['data']['chapters']):
            # `chapter_number` field can't be used, it's an internal sorting index
            data['chapters'].append({
                'slug': f'{chapter["id"]}:{chapter["slug"]}',  # noqa
                'title': chapter['name'],
                'scanlators': [chapter['group']] if chapter.get('group') else None,
                'date': convert_date_string(chapter['updated_at'].split('T')[0], format='%Y-%m-%d'),
            })

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns chapter data by scraping HTML page content

        Currently, only pages are expected.
        """
        _manga_id, manga_slug = manga_slug.split(':')
        chapter_id_, chapter_slug = chapter_slug.split(':')

        r = self.session_get(self.chapter_url.format(manga_slug, chapter_slug))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        images = None
        if script_element := soup.select_one('script#__NEXT_DATA__'):
            script = script_element.string
            if script:
                images = json.loads(script)['props']['pageProps']['initialChapter']['images']

        if images is None:
            return None

        data = dict(
            pages=[],
        )
        for index, image in enumerate(images, start=1):
            data['pages'].append(dict(
                slug=None,
                image=image,
                index=index,
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            page['image'],
            headers={
                'Referer': f'{self.base_url}/',
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name='{0:03d}.{1}'.format(page['index'], mime_type.split('/')[-1]),
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        _id, slug = slug.split(':')
        return self.manga_url.format(slug)

    def get_manga_list(self, term=None, content_rating=None, type=None, demographic=None, status=None, orderby=None):
        def get_page(page):
            params = {
                'limit': SEARCH_RESULTS_PAGE_LIMIT,
                'min_ch': 1,
                'page': page,
            }
            if term:
                params['q'] = term

            if content_rating:
                params['content_rating'] = content_rating
            if type:
                params['type'] = type
            if demographic:
                params['demographic'] = demographic
            if status:
                params['status'] = status

            if orderby == 'popular':
                params['sort'] = 'views'
            elif orderby == 'latest':
                params['sort'] = 'latest'

            r = self.session_get(
                self.api_search_url,
                params=params,
                headers={
                    'Referer': f'{self.search_url}',
                }
            )
            if r.status_code != 200:
                return [], False, None

            resp_data = r.json()
            if not resp_data.get('success'):
                return [], False, None

            more = resp_data['data']['pagination']['has_next'] and page < SEARCH_RESULTS_PAGES

            return resp_data['data']['items'], more, get_response_elapsed(r)

        results = []
        delay = None
        more = True
        page = 1
        while more:
            if delay:
                time.sleep(delay)

            items, more, rtime = get_page(page)
            for item in items:
                lastest_chapters = item.get('latest_chapters')

                results.append(dict(
                    slug=f'{item["id"]}:{item["slug"]}',  # noqa
                    name=item['name'],
                    cover=item['cover'],
                    last_chapter=lastest_chapters[0]['name'] if lastest_chapters else None,
                ))

            delay = min(rtime * 4, DOWNLOAD_MAX_DELAY) if rtime else None
            page += 1

        return results

    def get_latest_updates(self, content_rating=None, type=None, demographic=None, status=None):
        return self.get_manga_list(
            content_rating=content_rating, type=type, demographic=demographic, status=status, orderby='latest'
        )

    def get_most_populars(self, content_rating=None, type=None, demographic=None, status=None):
        return self.get_manga_list(
            content_rating=content_rating, type=type, demographic=demographic, status=status, orderby='popular'
        )

    def search(self, term, content_rating=None, type=None, demographic=None, status=None):
        return self.get_manga_list(
            term=term, content_rating=content_rating, type=type, demographic=demographic, status=status
        )



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from bs4 import BeautifulSoup

from komikku.servers import Server
from komikku.utils import get_buffer_mime_type
from komikku.webview import CompleteChallenge


class Scanmanga(Server):
    id = 'scanmanga'
    name = 'Scan Manga'
    lang = 'fr'
    is_nsfw = True
    long_strip_genres = ['Webcomic', ]
    status = 'disabled'  # 2024/03 chapters and chapters images URLs have become difficult to extract (no time for that)

    has_cf = True
    http_client = 'curl_cffi'

    base_url = 'https://www.scan-manga.com'
    latest_updates_url = base_url + '/?po'
    most_populars_url = base_url + '/TOP-Manga-Webtoon-25.html'
    api_search_url = base_url + '/api/search/quick.json'
    manga_url = base_url + '{0}'
    chapter_url = base_url + '/lecture-en-ligne/{0}{1}.html'
    cover_url = 'https://cdn.scanmanga.eu/img/manga/{0}'

    def __init__(self):
        self.session = None

    @classmethod
    def get_manga_initial_data_from_url(cls, url):
        return dict(
            url=url.replace(cls.base_url, ''),
            slug=url.split('/')[-1].replace('.html', ''),
        )

    @CompleteChallenge()
    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's relative url and slug (provided by search)
        """
        assert 'url' in initial_data and 'slug' in initial_data, 'Manga url or slug are missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['url']))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
        ))

        data['name'] = soup.find('div', class_='h2_titre').h2.text.strip()
        if data.get('cover') is None:
            data['cover'] = soup.find('div', class_='image_manga').img.get('src')

        # Details
        li_elements = soup.find('div', class_='contenu_texte_fiche_technique').find_all('li')
        for a_element in li_elements[0].find_all('a'):
            data['authors'].append(a_element.text.strip())

        data['genres'] = [g.strip() for g in li_elements[1].text.split()]
        for a_element in li_elements[2].find_all('a'):
            a_element.span.extract()
            data['genres'].append(a_element.text.strip())

        status = li_elements[6].text.strip().lower()
        if status == 'en cours':
            data['status'] = 'ongoing'
        elif status in ('one shot', 'terminé'):
            data['status'] = 'complete'
        elif status == 'en pause':
            data['status'] = 'hiatus'

        for a_element in li_elements[7].find_all('a'):
            data['scanlators'].append(a_element.text.strip())

        # Synopsis
        p_element = soup.find('div', class_='texte_synopsis_manga').find('p', itemprop='description')
        p_element.span.extract()
        data['synopsis'] = p_element.text.strip()

        # Chapters
        for element in reversed(soup.select('.chapitre_nom')):
            a_element = element.a
            if not a_element or element.select_one('.typcn-lock-closed') or element.select_one('.typcn-lock-open'):
                # Skip external chapters
                continue

            data['chapters'].append(dict(
                slug=a_element.get('href').split('/')[-1].replace(data['slug'], '').replace('.html', ''),
                title=element.text.strip(),
                date=None,
            ))

        return data

    @CompleteChallenge()
    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(self.chapter_url.format(manga_slug, chapter_slug))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = dict(
            pages=[],
        )

        for script_element in soup.find_all('script'):
            script = script_element.string
            if not script or 'var nPa = new Array' not in script:
                continue

            image_base_url = None
            for line in script.split('\n'):
                line = line.strip()

                if 'var nPa = new Array' in line:
                    array_name = line.split(' ')[1]

                    for item in line.split(';'):
                        if not item.startswith(f'{array_name}['):
                            continue

                        data['pages'].append(dict(
                            slug=None,
                            image=item.split('"')[1],
                        ))

                elif line.startswith(('tlo =', "$('#preload')")):
                    image_base_url = line.split("'")[-2]
                    break

            if image_base_url:
                for page in data['pages']:
                    page['image'] = image_base_url + page['image']

            break

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        self.create_session()  # Session must be refreshed each time

        r = self.session_get(
            page['image'],
            headers={
                'Referer': self.chapter_url.format(manga_slug, chapter_slug),
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('?')[0].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(url)

    @CompleteChallenge()
    def get_latest_updates(self):
        """
        Returns latest updates
        """
        r = self.session_get(self.latest_updates_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for a_element in soup.select('#content_news .nom_manga'):
            name = a_element.text.strip()
            if 'Novel' in name:
                continue

            results.append(dict(
                name=name,
                slug=a_element.get('href').split('/')[-1].replace('.html', ''),
                url=a_element.get('href').replace(self.base_url, ''),
            ))

        return results

    @CompleteChallenge()
    def get_most_populars(self):
        """
        Returns list of top manga
        """
        r = self.session_get(self.most_populars_url)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for cover_element in soup.select('.image_manga'):
            element = cover_element.find_next_siblings()[0]

            a_element = element.h3.a
            name = a_element.text.strip()
            if 'Novel' in name:
                # Skip
                continue

            results.append(dict(
                name=a_element.text.strip(),
                slug=a_element.get('href').split('/')[-1].replace('.html', ''),
                url=a_element.get('href').replace(self.base_url, ''),
                cover=cover_element.select_one('img').get('data-original'),
            ))

        return results

    @CompleteChallenge()
    def search(self, term):
        r = self.session_get(
            self.api_search_url,
            params=dict(term=term),
            default_headers=True,
            headers={
                'Accept': '*/*',
                'Content-Type': 'application/json; charset=UTF-8',
                'Referer': f'{self.base_url}/',
            }
        )

        if r.status_code == 200:
            try:
                data = r.json()
                results = []
                for item in data['title']:
                    if item['type'] == 'Novel':
                        # Skip
                        continue

                    results.append(dict(
                        url=item['url'],
                        slug=item['url'].split('/')[-1].replace('.html', ''),
                        name=item['nom_match'],
                        cover=self.cover_url.format(item['image']),
                        last_chapter=str(item['l_ch']),
                    ))
            except Exception:
                return None
            else:
                return results

        return None



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.foolslide import FoOlSlide


class Deathtollscans(FoOlSlide):
    id = 'deathtollscans'
    name = 'Death Toll Reader'
    lang = 'en'

    base_url = 'https://reader.deathtollscans.net'
    search_url = base_url + '/search/'
    manga_list_url = base_url + '/directory/'
    manga_url = base_url + '/series/{0}/'
    chapter_url = base_url + '/read/{0}/en/{1}/page/1'



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gettext import gettext as _
import logging
import re

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type
from komikku.webview import CompleteChallenge

logger = logging.getLogger(__name__)


class Madtheme(Server):
    base_url: str = None
    api_base_url: str = None
    api_chapters_url: str = None

    date_format = '%B %d, %Y'
    series_name = None

    search_results_selector = '.book-detailed-item'
    authors_selector = '.meta p:-soup-contains("Authors") a'
    status_selector = '.meta p:-soup-contains("Status") a'
    genres_selector = '.meta p:-soup-contains("Genres") a'
    synopsis_selector = 'p.content'
    chapters_selector = 'ul#chapter-list li > a'
    images_selector = '.chapter-image img'

    cover_src_attr = 'data-src'
    image_src_attr = 'data-src'

    filters = [
        {
            'key': 'status',
            'type': 'select',
            'name': _('Status'),
            'description': _('Filter by Status'),
            'value_type': 'single',
            'default': 'all',
            'options': [
                {'key': 'all', 'name': _('All')},
                {'key': 'ongoing', 'name': _('Ongoing')},
                {'key': 'complete', 'name': _('Completed')},
            ]
        },
    ]

    def __init__(self):
        self.search_url = self.base_url + '/search'
        if self.series_name:
            self.manga_url = f'{self.base_url}/{self.series_name}' + '/{0}'
            self.chapter_url = f'{self.base_url}/{self.series_name}' + '/{0}/{1}'
        else:
            self.manga_url = self.base_url + '/{0}'
            self.chapter_url = self.base_url + '/{0}/{1}'
        if not self.api_chapters_url:
            self.api_chapters_url = self.api_base_url + '/manga/{0}/chapters?source=detail'

        if self.session is None and not self.has_cf:
            self.session = requests.Session()
            self.session.headers.update({'User-Agent': USER_AGENT})

    @CompleteChallenge()
    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'

        r = self.session_get(
            self.manga_url.format(initial_data['slug'])
        )
        if r.status_code != 200:
            return None

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
        ))

        soup = BeautifulSoup(r.text, 'lxml')

        data['name'] = soup.select_one('h1').text.strip()
        data['cover'] = soup.select_one('#cover img').get(self.cover_src_attr)

        # Details
        for element in soup.select(self.authors_selector):
            data['authors'].append(element.text.replace(',', '').strip())

        if element := soup.select_one(self.status_selector):
            status = element.text.strip()
            data['status'] = 'complete' if status == 'Completed' else 'ongoing'

        for element in soup.select(self.genres_selector):
            data['genres'].append(element.text.replace(',', '').strip())

        if element := soup.select_one(self.synopsis_selector):
            data['synopsis'] = element.text.strip()

        # Chapters
        self.get_manga_chapters_data(data, soup)

        return data

    @CompleteChallenge()
    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(
            self.chapter_url.format(manga_slug, chapter_slug),
            headers={
                'Referer': self.manga_url.format(manga_slug),
            }
        )
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = dict(
            pages=[],
        )
        for element in soup.select(self.images_selector):
            data['pages'].append(dict(
                slug=None,
                image=element.get(self.image_src_attr),
            ))

        return data

    def get_manga_chapters_data(self, data, soup=None):
        """
        Get list of chapters
        """
        sid = self.get_manga_id(soup)
        if sid is None:
            logger.warning('Failed to get `%s` server Id', data['name'])
            return

        r = self.session_get(
            self.api_chapters_url.format(sid),
            headers={
                'Referer': self.manga_url.format(data['slug']),
            }
        )
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        for element in reversed(soup.select(self.chapters_selector)):
            title_element = element.select_one('.chapter-title')
            date_element = element.select_one('.chapter-update')

            data['chapters'].append({
                'slug': element.get('href').split('/')[-1],
                'title': title_element.text.strip(),
                'date': convert_date_string(date_element.text.strip(), format=self.date_format, languages=[self.lang]),
            })

    @CompleteChallenge()
    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            page['image'],
            headers={
                'Referer': f'{self.base_url}/',
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_id(self, soup):
        """
        Get manga server Id
        """
        sid = None
        for script_element in soup.find_all('script'):
            script = script_element.string
            if not script or 'var bookId' not in script:
                continue

            for line in script.split('\n'):
                line = line.strip()

                if matches := re.search(r'bookId = ([0-9]*);', script):
                    sid = matches.group(1)
                    break

            break

        return sid

    def get_manga_chapter_id(self, soup):
        """
        Get a manga chapter server Id
        """
        sid = None
        for script_element in soup.find_all('script'):
            script = script_element.string
            if not script or 'var chapterId' not in script:
                continue

            for line in script.split('\n'):
                line = line.strip()

                if matches := re.search(r'chapterId = ([0-9]*);', script):
                    sid = matches.group(1)
                    break

            break

        return sid

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_manga_list(self, term=None, status=None, orderby=None):
        params = {}
        if orderby == 'popular':
            params['sort'] = 'views'
        elif orderby == 'latest':
            params['sort'] = 'updated_at'
        else:
            params['q'] = term
        if status:
            params['status'] = status

        r = self.session_get(self.search_url, params=params)
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for item in soup.select(self.search_results_selector):
            a_element = item.select_one('.title a')
            a_thumb_element = item.select_one('.thumb a img')

            if element := a_thumb_element.select_one('latest-chapter'):
                last_chapter = element.text.strip()
            else:
                last_chapter = None

            results.append({
                'slug': a_element.get('href').split('/')[-1],
                'name': a_element.text.strip(),
                'cover': a_thumb_element.get(self.cover_src_attr),
                'last_chapter': last_chapter,
            })

        return results

    @CompleteChallenge()
    def get_latest_updates(self, status=None):
        """
        Returns list of latest manga
        """
        return self.get_manga_list(status=status, orderby='latest')

    @CompleteChallenge()
    def get_most_populars(self, status=None):
        """
        Returns list of popular manga
        """
        return self.get_manga_list(status=status, orderby='popular')

    @CompleteChallenge()
    def search(self, term, status=None):
        return self.get_manga_list(term=term, status=status)



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.manga_stream import MangaStream


class Mangastop(MangaStream):
    id = 'mangastop'
    name = 'Manga Stop'
    lang = 'pt_BR'
    is_nsfw = True

    has_cf = True

    base_url = 'https://mangastop.net'
    logo_url = 'https://images.mangastop.net/file/bucketamz/2025/09/logomarca-150x150.png'
    bypass_cf_url = base_url + '/manga/martial-peak/'

    authors_selector = '.j-manga-info p:-soup-contains("Artista") a, .j-manga-info p:-soup-contains("Autor") a'
    genres_selector = '.info-desc .mgen a, .tsinfo .imptdt:-soup-contains("Type") a'
    scanlators_selector = None
    status_selector = '.tsinfo .imptdt:-soup-contains("Status") i'
    synopsis_selector = '[itemprop="description"]'

    long_strip_genres = [
        'Manhua',
        'Manhwa',
    ]
